/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CFactoryObject.h"

CFactoryObject* CFactoryObject::me = NULL;

CFactoryObject* CFactoryObject::getInstance()
{
	if (me == NULL)
	{
		me = new CFactoryObject();
	}
	return me;
}

void CFactoryObject::release()
{
	delete me;
	me = NULL;
}

bool CFactoryObject::addCreator(ICreatorObject* creator)
{
	bool result = false;
	if (creator != NULL)
	{
		string name = creator->getCreationName();
		result = creators.insert(stringPICreatorObjectMap::value_type(name, creator)).second;
	}
	return result;
}

CObject* CFactoryObject::create(const string& typeName)
{
	CObject* object = NULL;
	stringPICreatorObjectMapIterator it = creators.find(typeName);
	if (it != creators.end())
	{
		ICreatorObject* creator = it->second;
		object = creator->create(typeName);
	}
	return object;
}

// This method is necessary in order to use the object factory.
// As the Engine3D is designed to be used as a library, the object
// creation system needs any kind of reference to a library method
// to create the "static" instances of all the creators. 
// In this method, all the creation object constructors are called, so
// realize that any class that want to use the creation system, has to 
// follow 3 steps:
//		1.- It needs to use the creation macros (see the CFactoryObject.h file)
//		2.- Needs a default constructor body in the .cpp file.
//		3.- Needs to construct an instance in this method.
#include "E3D_CTransform.h"
#include "E3D_CMesh.h"
#include "E3D_CCamera.h"
#include "E3D_CAnimationCurve.h"
#include "E3D_CShader.h"

void CFactoryObject::forceCreation()
{
	CTransform transform;
	CMesh mesh;
	CCamera camera;
	CAnimationCurve animationCurve;
	CShader shader;
}