/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CFACTORYOBJCET_H_
#define _E3D_CFACTORYOBJCET_H_

#pragma warning (disable: 4786 4503)

#include <string>
#include <map>
using std::map;
using std::string;

#define DEFINE_CREATOR_CLASS(CLASS)\
	class CCreator##CLASS: public ICreatorObject\
	{\
	public:\
		CCreator##CLASS();\
		string getCreationName();\
		CObject* create(const string& objectTypeName);\
	};

#define DEFINE_CREATOR_OBJECT(CLASS)\
	CCreator##CLASS::CCreator##CLASS()\
	{\
		CFactoryObject* factory = CFactoryObject::getInstance();\
		factory->addCreator(this);\
	}\
	string CCreator##CLASS::getCreationName()\
	{\
		return "" #CLASS "";\
	}\
	CObject* CCreator##CLASS::create(const string& objectTypeName)\
	{\
		CObject* object = NULL;\
		if (objectTypeName == "" #CLASS "")\
		{\
			object = new(##CLASS);\
		}\
		return object;\
	}\
	CCreator##CLASS creator;

class CObject;

class ICreatorObject
{
public:
	virtual CObject* create(const string& typeName) = 0;
	virtual string getCreationName() = 0;
};

typedef map<string, ICreatorObject*> stringPICreatorObjectMap;
typedef stringPICreatorObjectMap::iterator stringPICreatorObjectMapIterator;

class CFactoryObject
{
public:
	static void forceCreation();
	static CFactoryObject* getInstance();
	static void release();
	bool addCreator(ICreatorObject* creator);
	CObject* create(const string& objectTypeName);
protected:
	stringPICreatorObjectMap creators;
private:
	static CFactoryObject* me;
	CFactoryObject() { }
	~CFactoryObject() { }
};

#endif
