/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CObject_h_
#define _E3D_CObject_h_

#include <string>
using std::string;
#include <vector>
using std::vector;

#include "E3D_CFactoryObject.h"

// ============================================================================
// The main base class. All the classes inherit from it and contains some
// global and useful stuff like converting to a string or a method
// to know if two objects are equals... much more could come!
// ============================================================================
class CObject
{
public:
//Own
	// ============================================================================
	// In case that a derived object want to be deleted through a this class pointer/reference,
	// a virtual destructor will be needed.
	// ============================================================================
	virtual ~CObject();

	// ============================================================================
	// An operator to cast any object to a string. Polimorph it to give it a new
	// behaviour that fits your needs.
	// ============================================================================
	virtual operator string() const;

	// ============================================================================
	// An operator to compare the equality of two objects. Polimorph it to give
	// a new behaviour that fits your needs.
	// ============================================================================
	virtual bool operator==(const CObject& obj) const;

	// ============================================================================
	// A function to take all the data that an object needs from an string.
	// ============================================================================
	virtual bool fromString(const string& from);

	// ============================================================================
	// A function to take an object type string
	// ============================================================================
	string getTypeName();

protected:
	string typeName;
};

typedef vector<CObject*> pCObjectVector;
typedef pCObjectVector::iterator pCObjectVectorIterator;

#endif