/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CBoundingBox.h"
#include "E3D_CMatrix4D.h"

CBoundingBox::operator string() const
{
	char buf[256];
	sprintf(buf, "%s-%s", ((string)min).c_str(), ((string)max).c_str());
	return buf;
}

bool CBoundingBox::operator==(const CObject& obj) const
{
	bool result = false;
	try
	{
		const CBoundingBox& bb = static_cast<const CBoundingBox&>(obj);
		// Two bounding boxes are equals if their definitor vector are the same.
		result = min == bb.min && max == bb.max;
	}
	catch(bad_cast bce)
	{
	}
	return result;
}

CBoundingBox CBoundingBox::operator+(const CBoundingBox& bb) const
{
	// An addition between two bounding boxes means to get the biggest one.
	CBoundingBox res;

	if(min.x < bb.min.x)
		res.min.x = min.x;
	else
		res.min.x = bb.min.x;

	if(min.y < bb.min.y)
		res.min.y = min.y;
	else
		res.min.y = bb.min.y;

	if(min.z < bb.min.z)
		res.min.z = min.z;
	else
		res.min.z = bb.min.z;

	if(max.x < bb.max.x)
		res.max.x = bb.max.x;
	else
		res.max.x = max.x;

	if(max.y < bb.max.y)
		res.max.y = bb.max.y;
	else
		res.max.y = max.y;

	if(max.z < bb.max.z)
		res.max.z = bb.max.z;
	else
		res.max.z = max.z;

	return res;
}

void CBoundingBox::operator+=(const CBoundingBox& bb)
{
	// An addition between two bounding boxes means to get the biggest one
	if(min.x > bb.min.x)
		min.x = bb.min.x;

	if(min.y > bb.min.y)
		min.y = bb.min.y;

	if(min.z > bb.min.z)
		min.z = bb.min.z;

	if(max.x < bb.max.x)
		max.x = bb.max.x;

	if(max.y < bb.max.y)
		max.y = bb.max.y;

	if(max.z < bb.max.z)
		max.z = bb.max.z;
}

CVector3DVector CBoundingBox::getCorners() const
{
	CVector3DVector corners;

	corners.resize(8);

	// With the min and max vectors we can know the 8 corners of the bounding box
	corners[0].x = min.x;
	corners[0].y = min.y;
	corners[0].z = min.z;

	corners[1].x = max.x;
	corners[1].y = min.y;
	corners[1].z = min.z;

	corners[2].x = max.x;
	corners[2].y = max.y;
	corners[2].z = min.z;

	corners[3].x = min.x;
	corners[3].y = max.y;
	corners[3].z = min.z;

	corners[4].x = min.x;
	corners[4].y = min.y;
	corners[4].z = max.z;

	corners[5].x = max.x;
	corners[5].y = min.y;
	corners[5].z = max.z;

	corners[6].x = max.x;
	corners[6].y = max.y;
	corners[6].z = max.z;

	corners[7].x = min.x;
	corners[7].y = max.y;
	corners[7].z = max.z;

	return corners;
}
