/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CMath.h"
#include <math.h>

float CMath::sinus(float angle)
{
	float res = 0;
	if (!useRadians)
	{
		angle = DEGREE_TO_RADIAN(angle);
	}
	if (useTables)
	{
		int index = (int)(angle * RADIAN_TO_INDEX);
		index &= NUM_TABLE_ENTRIES - 1;
		res = sinTable[index];
	}
	else
	{
		res = (float)sin(angle);
	}
	return res;
}

float CMath::cosinus(float angle)
{
	float res = 0;
	if (!useRadians)
	{
		angle = DEGREE_TO_RADIAN(angle);
	}
	if (useTables)
	{
		int index = (int)(angle * RADIAN_TO_INDEX);
		index &= NUM_TABLE_ENTRIES - 1;
		res = cosTable[index];
	}
	else
	{
		res = (float)cos(angle);
	}
	return res;
}

float CMath::tangent(float angle)
{
	float res = 0;
	if (!useRadians)
	{
		angle = DEGREE_TO_RADIAN(angle);
	}
	if (useTables)
	{
		int index = (int)(angle * RADIAN_TO_INDEX);
		index &= NUM_TABLE_ENTRIES - 1;
		res = tanTable[index];
	}
	else
	{
		res = (float)tan(angle);
	}
	return res;
}

void CMath::setUseTables(bool ut)
{
	createTables();
	useTables = ut;
}

void CMath::setUseRadians(bool ur)
{
	useRadians = ur;
}

bool CMath::getUseTables()
{
	return useTables;
}

bool CMath::getUseRadians()
{
	return useRadians;
}

const float CMath::PI = 3.1415926535897932384626433832795f;
const float CMath::TWO_PI = CMath::PI * 2;
const float CMath::PI_DIV_TWO = CMath::PI / 2;

void CMath::createTables()
{
	if (tablesCreated) return;
	float angle;
	for (int i=0; i<NUM_TABLE_ENTRIES; i++)
	{
		angle = i / RADIAN_TO_INDEX;
		sinTable[i] = (float)sin(angle);
		cosTable[i] = (float)cos(angle);
		tanTable[i] = sinTable[i] / cosTable[i];
	}
	tablesCreated = true;
}

bool CMath::useTables = false;
bool CMath::useRadians = true;
bool CMath::tablesCreated = false;
float CMath::sinTable[NUM_TABLE_ENTRIES];
float CMath::cosTable[NUM_TABLE_ENTRIES];
float CMath::tanTable[NUM_TABLE_ENTRIES];

const float CMath::RADIAN_TO_INDEX = NUM_TABLE_ENTRIES / CMath::TWO_PI;
