/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CMatrix4D.h"
#include "E3D_CVector3D.h"
#include "E3D_CMath.h"

CMatrix4D::operator string() const
{
	char buf[256];
	sprintf(buf, "{\n%s\n%s\n%s\n%s\n}", ((string)x).c_str(), ((string)y).c_str(), ((string)z).c_str(), ((string)w).c_str());
	return buf;
}

bool CMatrix4D::operator==(const CObject& obj) const
{
	bool result = false;
	try
	{
		const CMatrix4D& m = static_cast<const CMatrix4D&>(obj);
		result = (x == m.x) && (y == m.y) && (z == m.z) && (w == m.w);
	}
	catch(bad_cast bce)
	{
	}
	return result;
}

CMatrix4D CMatrix4D::operator+(const CMatrix4D& m) const
{
	CMatrix4D res;
	res.x.x = x.x + m.x.x;	res.y.x = y.x + m.y.x;	res.z.x = z.x + m.z.x;	res.w.x = w.x + m.w.x;
	res.x.y = x.y + m.x.y;	res.y.y = y.y + m.y.y;	res.z.y = z.y + m.z.y;	res.w.y = w.y + m.w.y;
	res.x.z = x.z + m.x.z;	res.y.z = y.z + m.y.z;	res.z.z = z.z + m.z.z;	res.w.z = w.z + m.w.z;
	res.x.w = x.w + m.x.w;	res.y.w = y.w + m.y.w;	res.z.w = z.w + m.z.w;	res.w.w = w.w + m.w.w;
	return res;
}

CMatrix4D CMatrix4D::operator-(const CMatrix4D& m) const
{
	CMatrix4D res;
	res.x.x = x.x - m.x.x;	res.y.x = y.x - m.y.x;	res.z.x = z.x - m.z.x;	res.w.x = w.x - m.w.x;
	res.x.y = x.y - m.x.y;	res.y.y = y.y - m.y.y;	res.z.y = z.y - m.z.y;	res.w.y = w.y - m.w.y;
	res.x.z = x.z - m.x.z;	res.y.z = y.z - m.y.z;	res.z.z = z.z - m.z.z;	res.w.z = w.z - m.w.z;
	res.x.w = x.w - m.x.w;	res.y.w = y.w - m.y.w;	res.z.w = z.w - m.z.w;	res.w.w = w.w - m.w.w;
	return res;
}

CBoundingBox CMatrix4D::operator*(const CBoundingBox& bb) const
{
	CBoundingBox res;

	// First of all we get all the corners of out current bounding box
	CVector3DVector corners = const_cast<CBoundingBox&>(bb).getCorners();

	// We transform them all to know their real positions in space
	corners = (*this) * corners;

	// At the end we only need the minimun and maximun vectors for the bounding box
	// so, we check to find them
	res.min = CVector3D(1e9f, 1e9f, 1e9f);
	res.max = CVector3D(-1e9f, -1e9f, -1e9f);

	for(int i = 0; i < 8; i++)
	{
		if(res.min.x > corners[i].x)
			res.min.x = corners[i].x;

		if(res.min.y > corners[i].y)
			res.min.y = corners[i].y;

		if(res.min.z > corners[i].z)
			res.min.z = corners[i].z;

		if(res.max.x < corners[i].x)
			res.max.x = corners[i].x;

		if(res.max.y < corners[i].y)
			res.max.y = corners[i].y;

		if(res.max.z < corners[i].z)
			res.max.z = corners[i].z;
	}

	return res;
}

CMatrix4D CMatrix4D::operator*(const CMatrix4D& m) const
{
	CMatrix4D res;
	res.x.x = (x.x * m.x.x) + (y.x * m.x.y) + (z.x * m.x.z) + (w.x * m.x.w);
	res.y.x = (x.x * m.y.x) + (y.x * m.y.y) + (z.x * m.y.z) + (w.x * m.y.w);
	res.z.x = (x.x * m.z.x) + (y.x * m.z.y) + (z.x * m.z.z) + (w.x * m.z.w);
	res.w.x = (x.x * m.w.x) + (y.x * m.w.y) + (z.x * m.w.z) + (w.x * m.w.w);

	res.x.y = (x.y * m.x.x) + (y.y * m.x.y) + (z.y * m.x.z) + (w.y * m.x.w);
	res.y.y = (x.y * m.y.x) + (y.y * m.y.y) + (z.y * m.y.z) + (w.y * m.y.w);
	res.z.y = (x.y * m.z.x) + (y.y * m.z.y) + (z.y * m.z.z) + (w.y * m.z.w);
	res.w.y = (x.y * m.w.x) + (y.y * m.w.y) + (z.y * m.w.z) + (w.y * m.w.w);

	res.x.z = (x.z * m.x.x) + (y.z * m.x.y) + (z.z * m.x.z) + (w.z * m.x.w);
	res.y.z = (x.z * m.y.x) + (y.z * m.y.y) + (z.z * m.y.z) + (w.z * m.y.w);
	res.z.z = (x.z * m.z.x) + (y.z * m.z.y) + (z.z * m.z.z) + (w.z * m.z.w);
	res.w.z = (x.z * m.w.x) + (y.z * m.w.y) + (z.z * m.w.z) + (w.z * m.w.w);

	res.x.w = (x.w * m.x.x) + (y.w * m.x.y) + (z.w * m.x.z) + (w.w * m.x.w);
	res.y.w = (x.w * m.y.x) + (y.w * m.y.y) + (z.w * m.y.z) + (w.w * m.y.w);
	res.z.w = (x.w * m.z.x) + (y.w * m.z.y) + (z.w * m.z.z) + (w.w * m.z.w);
	res.w.w = (x.w * m.w.x) + (y.w * m.w.y) + (z.w * m.w.z) + (w.w * m.w.w);
	return res;
}

CVector3D CMatrix4D::operator*(const CVector3D& v) const
{
	CVector3D res;
	res.x = (x.x * v.x) + (y.x * v.y) + (z.x * v.z) + w.x;
	res.y = (x.y * v.x) + (y.y * v.y) + (z.y * v.z) + w.y;
	res.z = (x.z * v.x) + (y.z * v.y) + (z.z * v.z) + w.z;
	return res;
}

CVector4D CMatrix4D::operator*(const CVector4D& v) const
{
	CVector4D res;
	res.x = (x.x * v.x) + (y.x * v.y) + (z.x * v.z) + (w.x * v.w);
	res.y = (x.y * v.x) + (y.y * v.y) + (z.y * v.z) + (w.y * v.w);
	res.z = (x.z * v.x) + (y.z * v.y) + (z.z * v.z) + (w.z * v.w);
	res.w = (x.w * v.x) + (y.w * v.y) + (z.w * v.z) + (w.w * v.w);
	return res;
}

CMatrix4D CMatrix4D::operator*(float factor) const
{
	CMatrix4D res;
	res.x.x = x.x * factor;		res.y.x = y.x * factor;		res.z.x = z.x * factor;		res.w.x = w.x * factor;
	res.x.y = x.y * factor;		res.y.y = y.y * factor;		res.z.y = z.y * factor;		res.w.y = w.y * factor;
	res.x.z = x.z * factor;		res.y.z = y.z * factor;		res.z.z = z.z * factor;		res.w.z = w.z * factor;
	res.x.w = x.w * factor;		res.y.w = y.w * factor;		res.z.w = z.w * factor;		res.w.w = w.w * factor;
	return res;
}

CVector3DVector CMatrix4D::operator*(const CVector3DVector& vectors) const
{
	CVector3DVector result = vectors;
	// Transform every vector by this transform matrix
	CVector3DVectorIterator it = result.begin();
	for(; it != result.end(); it++)
	{
		CVector3D& v3D = *it;
		v3D = *this * v3D;
	}
	return result;
}

void CMatrix4D::translateRotateScale(const CVector3D& t, const CVector3D& r, const CVector3D& s)
{
	/*
	Rotate in X:

	z' = y * cx - z * sx
	y' = y * sx + z * cx

	1		0		0		0
	0		cx	-sx	0
	0		sx	cx	0
	0		0		0		1

	Rotate in Y:

	z' = x * cy + z * sy
	x' = -x * sy + z * cy

	cy	0		sy	0
	0		1		0		0
	-sy	0		cy	0
	0		0		0		1

	Rotate in Z:

	x' = x * cz - y * sz
	y' = x * sz + y * cz

	cz	-sz	0		0
	sz	cz	0		0
	0		0		1		0
	0		0		0		1
	*/

	// Multiply the three rotation matrices Rz * Ry * Rx and you'll get all these... 
	// remember that w is the homogeneous component that indicates the translation
	float sx = (float)CMath::sinus(r.x);
	float cx = (float)CMath::cosinus(r.x);
	float sy = (float)CMath::sinus(r.y);
	float cy = (float)CMath::cosinus(r.y);
	float sz = (float)CMath::sinus(r.z);
	float cz = (float)CMath::cosinus(r.z);

	x.x = cz * cy;		y.x = cz * sy * sx - sz * cx;		z.x = cz * sy * cx + sz * sx;		w.x = t.x;
	x.y = sz * cy;		y.y = sz * sy * sx + cz * cx;		z.y = sz * sy * cx - cz * sx;	  w.y = t.y;
	x.z = -sy;				y.z = cy * sx;									z.z = cy * cx;									w.z = t.z;
	x.w = 0.0f;       y.w = 0.0f;       							z.w = 0.0f;								     	w.w = 1.0f;		
	
	x *= s.x;
	y *= s.y;
	z *= s.z;
}

void CMatrix4D::perspectiveProjection(float l, float r, float b, float t, float zn, float zf)
{
	x.x = 2.0f / (r - l);
	y.x = 0.0f;
	z.x = (r + l) / (r - l);
	w.x = 0.0f;

	x.y = 0.0f;
	y.y = 2.0f / (t - b);
	z.y = (t + b) / (t - b);
	w.y = 0.0f;

	x.z = 0.0f;
	y.z = 0.0f;
	z.z = -(zf + zn) / (zf - zn);
	w.z = (2.0f * zf * zn) / (zf - zn);

	x.w = 0.0f;
	y.w = 0.0f;
	z.w = -1.0f;
	w.w = 0.0f;
}

void CMatrix4D::orthographicProjection(float l, float r, float b, float t, float zn, float zf)
{
	x.x = 2.0f / (r - l);
	y.x = 0.0f;
	z.x = 0.0f;
	w.x = -(r + l) / (r - l);

	x.y = 0.0f;
	y.y = 2.0f / (t - b);
	z.y = 0.0f;
	w.y = -(t + b) / (t - b);

	x.z = 0.0f;
	y.z = 0.0f;
	z.z = -2.0f / (zf - zn);
	w.z = -(zf + zn) / (zf - zn);

	x.w = 0.0f;
	y.w = 0.0f;
	z.w = 0.0f;
	w.w = 1.0f;
}

CMatrix4D CMatrix4D::inverse(const CMatrix4D& m)
{
	CMatrix4D res;
	float det;

	det = m.x.x * (m.y.y * m.z.z - m.y.z * m.z.y) - 
				m.y.x * (m.x.y * m.z.z - m.x.z * m.z.y) + 
				m.z.x * (m.x.y * m.y.z - m.x.z * m.y.y);

	res.x.x =  (m.y.y * m.z.z - m.y.z * m.z.y);		res.y.x = -(m.y.x * m.z.z - m.y.z * m.z.x);		res.z.x =  (m.y.x * m.z.y - m.y.y * m.z.x);		res.w.x = 0.0f;
	res.x.y = -(m.x.y * m.z.z - m.x.z * m.z.y);		res.y.y =  (m.x.x * m.z.z - m.x.z * m.z.x);		res.z.y = -(m.x.x * m.z.y - m.x.y * m.z.x);		res.w.x = 0.0f;
	res.x.z =  (m.x.y * m.y.z - m.x.z * m.y.y);		res.y.z = -(m.x.x * m.y.z - m.x.z * m.y.x);		res.z.z =  (m.x.x * m.y.y - m.x.y * m.y.x);		res.w.x = 0.0f;
	res.x.w = 0.0f;																res.y.w = 0.0f;																res.z.w = 0.0f;																res.w.w = 1.0f;

	res = (1.0f / det) * res;

	res.w.x = -((res.x.x * m.w.x) + (res.y.x * m.w.y) + (res.z.x * m.w.z));
	res.w.y = -((res.x.y * m.w.x) + (res.y.y * m.w.y) + (res.z.y * m.w.z));
	res.w.z = -((res.x.z * m.w.x) + (res.y.z * m.w.y) + (res.z.z * m.w.z));
	res.w.w = 1.0f;

	return res;
}

void CMatrix4D::identity()
{
	zero();
	x.x = y.y = z.z = w.w = 1;
}

void CMatrix4D::zero()
{
	x = y = z = w = CVector4D();
}

bool CMatrix4D::toArray(float* array) const
{
	if (array == NULL) return false;
	array[0] = x.x; array[4] = y.x; array[8]  =	z.x; array[12] = w.x;
	array[1] = x.y; array[5] = y.y; array[9]  =	z.y; array[13] = w.y;
	array[2] = x.z; array[6] = y.z; array[10] =	z.z; array[14] = w.z;
	array[3] = x.w; array[7] = y.w; array[11] =	z.w; array[15] = w.w;
	return true;
}

void CMatrix4D::fromArray(float* array)
{
	if (array == NULL) return;
	x.x = array[0]; y.x = array[4]; z.x = array[8];  w.x = array[12];
	x.y = array[1]; y.y = array[5]; z.y = array[9];  w.y = array[13];
	x.z = array[2]; y.z = array[6]; z.z = array[10]; w.z = array[14];
	x.w = array[3]; y.w = array[7]; z.w = array[11]; w.w = array[15];
}

CMatrix4D operator*(float factor, const CMatrix4D& m)
{
	CMatrix4D res;
	res.x.x = m.x.x * factor;		res.y.x = m.y.x * factor;		res.z.x = m.z.x * factor;		res.w.x = m.w.x * factor;
	res.x.y = m.x.y * factor;		res.y.y = m.y.y * factor;		res.z.y = m.z.y * factor;		res.w.y = m.w.y * factor;
	res.x.z = m.x.z * factor;		res.y.z = m.y.z * factor;		res.z.z = m.z.z * factor;		res.w.z = m.w.z * factor;
	res.x.w = m.x.w * factor;		res.y.w = m.y.w * factor;		res.z.w = m.z.w * factor;		res.w.w = m.w.w * factor;
	return res;
}

