/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CVector3D.h"
#include "E3D_CVector4D.h"

#include <stdio.h>

#include <sstream>
using std::stringstream;

CVector3D::operator string() const
{
	char buf[256];
	sprintf(buf, "(%f, %f, %f)", x, y, z);
	return buf;
}

bool CVector3D::operator==(const CObject& obj) const
{
	bool result = false;
	try
	{
		const CVector3D& v3D = static_cast<const CVector3D&>(obj);
		result = (x == v3D.x) && (y == v3D.y) && (z == v3D.z);
	}
	catch(bad_cast e)
	{
	}
	return result;
}

bool CVector3D::fromString(const string& from)
{
	bool result = false;
	int index;
	string temp;
	string value;
	temp = from;
	index = temp.find_first_of("(");
	if (index != temp.npos)
	{
		temp = temp.substr(index + 1, temp.length() - index - 1);
		if ((index = temp.find_first_of(", ")) != temp.npos)
		{
			value = temp.substr(0, index);
			stringstream toX(value);
			toX >> x;
			temp = temp.substr(index + 2, temp.length() - index - 2);
			if ((index = temp.find_first_of(", ")) != temp.npos)
			{
				value = temp.substr(0, index);
				stringstream toY(value);
				toY >> y;
				temp = temp.substr(index + 2, temp.length() - index - 2);
				if ((index = temp.find_first_of(")")) != temp.npos)
				{
					value = temp.substr(0, index);
					stringstream toZ(value);
					toZ >> z;
					result = true;
				}		
			}		
		}		
	}
	return result;
}

CVector3D::CVector3D()
{
	x = y = z = 0;
}

CVector3D::CVector3D(float x, float y, float z): x(x), y(y), z(z)
{
}

CVector3D CVector3D::operator+(const CVector3D& v) const
{
	CVector3D res;
	res.x = x + v.x;
	res.y = y + v.y;
	res.z = z + v.z;
	return res;
}

void CVector3D::operator+=(const CVector3D& v)
{
	x += v.x;
	y += v.y;
	z += v.z;
}

CVector3D CVector3D::operator-(const CVector3D& v) const
{
	CVector3D res;
	res.x = x - v.x;
	res.y = y - v.y;
	res.z = z - v.z;
	return res;
}

void CVector3D::operator-=(const CVector3D& v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
}

CVector3D CVector3D::operator-()
{
	x = -x;
	y = -y;
	z = -z;
	return *this;
}

CVector3D CVector3D::operator*(const CVector3D& v) const
{
	CVector3D res;
	res.x = x * v.x;
	res.y = y * v.y;
	res.z = z * v.z;
	return res;
}

CVector3D CVector3D::operator*(float factor) const
{
	CVector3D res;
	res.x = x * factor;
	res.y = y * factor;
	res.z = z * factor;
	return res;
}

void CVector3D::operator*=(const CVector3D& v)
{
	x *= v.x;
	y *= v.y;
	z *= v.z;
}

void CVector3D::operator*=(float factor)
{
	x *= factor;
	y *= factor;
	z *= factor;
}

CVector3D CVector3D::operator/(const CVector3D& v) const
{
	CVector3D res;
	res.x = (v.x != 0) ? (x / v.x) : (x);
	res.y = (v.y != 0) ? (y / v.y) : (y);
	res.z = (v.z != 0) ? (z / v.z) : (z);
	return res;
}

CVector3D CVector3D::operator/(float factor) const
{
	CVector3D res;
	if (factor != 0)
	{
		res.x = x / factor;
		res.y = y / factor;
		res.z = z / factor;
	}
	else
	{
		res = *this;
	}
	return res;
}

void CVector3D::operator/=(const CVector3D& v)
{
	if (v.x != 0)	x /= v.x;
	if (v.y != 0) y /= v.y;
	if (v.z != 0) z /= v.z;
}

void CVector3D::operator/=(float factor)
{
	if (factor != 0)
	{
		x /= factor;
		y /= factor;
		z /= factor;
	}
}

CVector3D CVector3D::operator%(const CVector3D& v) const
{
	CVector3D res;
	res.x = (y * v.z) - (z * v.y);
	res.y = (z * v.x) - (x * v.z);
	res.z = (x * v.y) - (y * v.x);
	return res;
}

CVector4D CVector3D::operator%(const CVector4D& v) const
{
	CVector4D res;
	res.x = (y * v.z) - (z * v.y);
	res.y = (z * v.x) - (x * v.z);
	res.z = (x * v.y) - (y * v.x);
	res.w = 1.0f;
	return res;
}

float CVector3D::operator^(const CVector3D& v) const
{
	float res;
	res = (x * v.x) + (y * v.y) + (z * v.z);
	return res;
}

CVector3D::operator CVector4D() const
{
	CVector4D res;
	res.x = this->x;
	res.y = this->y;
	res.z = this->z;
	res.w = 1;
	return res;
}

CVector3D CVector3D::operator=(const CVector4D& v)
{
	this->x = v.x;
	this->y = v.y;
	this->z = v.z;
	return *this;
}

CVector3D operator*(float factor, const CVector3D& v)
{
	CVector3D res;
	res.x = factor * v.x;
	res.y = factor * v.y;
	res.z = factor * v.z;
	return res;
}

CVector3D operator/(float factor, const CVector3D& v)
{
	CVector3D res;
	res.x = (v.x != 0) ? (factor / v.x) : (v.x);
	res.y = (v.y != 0) ? (factor / v.y) : (v.y);
	res.z = (v.z != 0) ? (factor / v.z) : (v.z);
	return res;
}
