/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CVector4D.h"
#include "E3D_CVector3D.h"

CVector4D::operator string() const
{
	char buf[256];
	sprintf(buf, "(%f, %f, %f, %f)", x, y, z, w);
	return buf;
}

bool CVector4D::operator==(const CObject& obj) const
{
	bool result = false;
	try
	{
		const CVector4D& v4D = static_cast<const CVector4D&>(obj);
		result = (x == v4D.x) && (y == v4D.y) && (z == v4D.z) && (w == v4D.w);
	}
	catch(bad_cast bce)
	{
	}
	return result;
}

CVector4D::CVector4D()
{
	x = y = z = w = 0;
}

CVector4D::CVector4D(float x, float y, float z, float w)
{
	this->x = x;
	this->y = y;
	this->z = z;
	this->w = w;
}

CVector4D CVector4D::operator+(const CVector4D& v) const
{
	CVector4D res;
	res.x = x + v.x;
	res.y = y + v.y;
	res.z = z + v.z;
	res.w = w + v.w;
	return res;
}

void CVector4D::operator+=(const CVector4D& v)
{
	x += v.x;
	y += v.y;
	z += v.z;
	w += v.w;
}

CVector4D CVector4D::operator-(const CVector4D& v) const
{
	CVector4D res;
	res.x = x - v.x;
	res.y = y - v.y;
	res.z = z - v.z;
	res.w = w - v.w;
	return res;
}

void CVector4D::operator-=(const CVector4D& v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
	w -= v.w;
}

CVector4D CVector4D::operator-()
{
	x = -x;
	y = -y;
	z = -z;
	w = -w;
	return *this;
}

CVector4D CVector4D::operator*(const CVector4D& v) const
{
	CVector4D res;
	res.x = x * v.x;
	res.y = y * v.y;
	res.z = z * v.z;
	res.w = w * v.w;
	return res;
}

CVector4D CVector4D::operator*(float factor) const
{
	CVector4D res;
	res.x = x * factor;
	res.y = y * factor;
	res.z = z * factor;
	res.w = w * factor;
	return res;
}

void CVector4D::operator*=(const CVector4D& v)
{
	x *= v.x;
	y *= v.y;
	z *= v.z;
	w *= v.w;
}

void CVector4D::operator*=(float factor)
{
	x *= factor;
	y *= factor;
	z *= factor;
	w *= factor;
}

CVector4D CVector4D::operator/(const CVector4D& v) const
{
	CVector4D res;
	res.x = (v.x != 0) ? (x / v.x) : (x);
	res.y = (v.y != 0) ? (y / v.y) : (y);
	res.z = (v.z != 0) ? (z / v.z) : (z);
	res.w = w;
	return res;
}

CVector4D CVector4D::operator/(float factor) const
{
	CVector4D res;
	if (factor != 0)
	{
		res.x = x / factor;
		res.y = y / factor;
		res.z = z / factor;
	}
	else
	{
		res = *this;
	}
	res.w = w;
	return res;
}

void CVector4D::operator/=(const CVector4D& v)
{
	if (v.x != 0)	x /= v.x;
	if (v.y != 0) y /= v.y;
	if (v.z != 0) z /= v.z;
}

void CVector4D::operator/=(float factor)
{
	if (factor != 0)
	{
		x /= factor;
		y /= factor;
		z /= factor;
	}
}

CVector4D CVector4D::operator%(const CVector4D& v) const
{
	CVector4D res;
	res.x = (y * v.z) - (z * v.y);
	res.y = (z * v.x) - (x * v.z);
	res.z = (x * v.y) - (y * v.x);
	res.w = 1.0f;
	return res;
}

CVector4D CVector4D::operator%(const CVector3D& v) const
{
	CVector4D res;
	res.x = (y * v.z) - (z * v.y);
	res.y = (z * v.x) - (x * v.z);
	res.z = (x * v.y) - (y * v.x);
	res.w = 1.0f;
	return res;
}

float CVector4D::operator^(const CVector4D& v) const
{
	float res;
	res = x * v.x + y * v.y + z * v.z;
	return res;
}

float CVector4D::operator^(const CVector3D& v) const
{
	float res;
	res = x * v.x + y * v.y + z * v.z + w;
	return res;
}

CVector4D::operator CVector3D() const
{
	return CVector3D(x, y, z);
}

CVector4D CVector4D::operator=(const CVector3D& v)
{
	this->x = v.x;
	this->y = v.y;
	this->z = v.z;
	return *this;
}

CVector4D operator*(float factor, const CVector4D& v)
{
	CVector4D res;
	res.x = factor * v.x;
	res.y = factor * v.y;
	res.z = factor * v.z;
	res.w = v.w;
	return res;
}

CVector4D operator/(float factor, const CVector4D& v)
{
	CVector4D res;
	res.x = (v.x != 0) ? (factor / v.x) : (v.x);
	res.y = (v.y != 0) ? (factor / v.y) : (v.y);
	res.z = (v.z != 0) ? (factor / v.z) : (v.z);
	res.w = v.w;
	return res;
}


