/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CBoundingBox_h_
#define _E3D_CBoundingBox_h_

// This class stores the data related with an imaginary bounding box. All you need to
// define this kind of 3D structure is to have 2 vectors: Min and Max. From
// them you can find out all the 8 corners of the box.

#include "E3D_CObject.h"
#include "E3D_CVector3D.h"

// ============================================================================
// A class that stores the necesary data to define an imaginary box. It is
// going to be used for non exact collision detection, culling etc.
// ============================================================================
class CBoundingBox: public CObject
{
public:
	// From CObject
	virtual operator string() const;
	virtual bool operator==(const CObject& obj) const;

// Own
	// ============================================================================
	// Adds two bounding boxes. This means to create a new bounding box
	// that contains both of the operands (this and bb).
	// ============================================================================
	CBoundingBox operator+(const CBoundingBox& bb) const;
	// ============================================================================
	// Adds two bounding boxes leaving the result inside one of the operands
	// (this), so it will contain both of them.
	// ============================================================================
	void operator+=(const CBoundingBox& bb);
	// ============================================================================
	// Gets a vector with all of the 8 corners that define a bounding box.
	// All the corners can be reached with the 2 vectors (min & max).
	// ============================================================================
	CVector3DVector getCorners() const;

	CVector3D min, max;
};

#endif