/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CMATH_H_
#define _E3D_CMATH_H_

#include "E3D_CObject.h"
#include "math.h"

#define NUM_TABLE_ENTRIES 65535
#define DEGREE_TO_RADIAN(angle) ((CMath::PI * angle * PI) / 180)

// ============================================================================
// This class encapsulates all the stuff related with mathematical operations.
// Most of the methods are static and public so can be addressed anytime from anywhere.
// ============================================================================
class CMath: public CObject
{
public:

// Own
	// ============================================================================
	// Calc the sinus of the angle
	// ============================================================================
	static float sinus(float angle);

	// ============================================================================
	// Calc the cosinus of the angle
	// ============================================================================
	static float cosinus(float angle);

	// ============================================================================
	// Calc the tangent of the angle
	// ============================================================================
	static float tangent(float angle);

	// ============================================================================
	// Indicate to use or not tables to accelerate the process
	// ============================================================================
	static void setUseTables(bool ut);

	// ============================================================================
	// Indicate to use either radians or degrees
	// ============================================================================
	static void setUseRadians(bool ur);

	// ============================================================================
	// Ask if the class is using tables or not.
	// ============================================================================
	static bool getUseTables();

	// ============================================================================
	// Ask if the class is using radians or degrees
	// ============================================================================
	static bool getUseRadians();

	static const float PI;
	static const float TWO_PI;
	static const float PI_DIV_TWO;

protected:
	static void createTables();

	static const float RADIAN_TO_INDEX;

	static bool useTables;
	static bool useRadians;
	static bool tablesCreated;
	static float sinTable[NUM_TABLE_ENTRIES];
	static float cosTable[NUM_TABLE_ENTRIES];
	static float tanTable[NUM_TABLE_ENTRIES];

};

#endif