/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CRenderWOGL.h"
#include "E3D_CMesh.h"
#include "E3D_CBoundingBox.h"

#include <gl/glut.h>

bool CRenderWOGL::init(void* window, int width, int height, bool fullscreen)
{
	this->width = width;
	this->height = height;

	if (window == NULL)
	{
		useGLUT = true;
		glutInitWindowSize(width, height);
	}
	else
	{
		hwnd = (HWND)window;
		hdc = GetDC(hwnd);

		MoveWindow(hwnd, 0, 0, width, height, true);

		PIXELFORMATDESCRIPTOR pfd;
		memset(&pfd, 0, sizeof(PIXELFORMATDESCRIPTOR));
		pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);   
		pfd.nVersion = 1;
		pfd.dwFlags = PFD_DOUBLEBUFFER | PFD_SUPPORT_OPENGL | PFD_DRAW_TO_WINDOW;
		pfd.iLayerType = PFD_MAIN_PLANE;
		pfd.iPixelType = PFD_TYPE_RGBA;
		pfd.cColorBits = 32;
		int pf = ChoosePixelFormat(hdc, &pfd);
		SetPixelFormat(hdc, pf, &pfd);
		context = wglCreateContext(hdc);
		wglMakeCurrent(hdc, context);
	}

	glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH);
	glViewport(0, 0, width, height);
 	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, (GLfloat)width / (GLfloat)height, 1, 1000);
	glMatrixMode(GL_MODELVIEW);

//	glEnable(GL_LIGHTING);
//	glEnable(GL_LIGHT0);

//	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glClearColor(0.5, 0.5, 0.5, 0);

	return true;
}

bool CRenderWOGL::end()
{
	wglMakeCurrent(NULL, NULL);

	if (context != NULL)
	{
		wglDeleteContext(context);
		context = NULL;
	}
	return true;
}

bool CRenderWOGL::startFrame()
{
	// Borrar el buffer y el zbuffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	return true;
}

bool CRenderWOGL::flip()
{
	if (useGLUT)
	{
		glutSwapBuffers();
	}
	else
	{
		SwapBuffers(hdc);
	}
	glFlush();
	return true;
}

bool CRenderWOGL::endFrame()
{
	return true;
}

bool CRenderWOGL::setViewMatrix(float* viewMatrix)
{
	this->viewMatrix = viewMatrix;
	return true;
}

bool CRenderWOGL::pushModelMatrix(float* modelMatrix)
{
	glPushMatrix();
	glLoadMatrixf(viewMatrix);
	glMultMatrixf(modelMatrix);
	return false;
}

bool CRenderWOGL::popModelMatrix()
{
	glPopMatrix();
	return true;
}

bool CRenderWOGL::renderModel()
{
	int i;

	// Positions...
	if (model.positions != NULL)
	{
		glVertexPointer(3, GL_FLOAT, 3 * sizeof(float), (const GLvoid *)(model.positions));
		glEnableClientState(GL_VERTEX_ARRAY);
	}

	// Normals...
	if (model.normals != NULL)
	{
		glNormalPointer(GL_FLOAT, 3 * sizeof(float), (const GLvoid *)(model.normals));
		glEnableClientState(GL_NORMAL_ARRAY);
	}

	// Texture coordinates...
	if (model.uvCoords != NULL)
	{
		glTexCoordPointer(2, GL_FLOAT, sizeof(float) * 2, (const GLvoid*)(model.uvCoords));
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	}

	// Colors...
	if (model.colors != NULL)
	{
		glColorPointer(4, GL_FLOAT, 4 * sizeof(float), (const GLvoid *)(model.colors));
		glEnableClientState(GL_COLOR_ARRAY);
	}

	// Get the correct render mode for this mesh from E3D to oGL
	GLenum type = E3D_MODE_TO_GL_MODE(model.mode);

	// Render the arrays
	for (i=0; i<model.numIndices; i++)
	{
		glDrawArrays(type, (GLint)(model.indices[i].start), (GLint)(model.indices[i].end));
	}
	return true;
}

bool CRenderWOGL::renderBoundingBox(const CBoundingBox& boundingBox)
{
	glPushMatrix();
	glLoadMatrixf(viewMatrix);
	CVector3DVector corners = boundingBox.getCorners();
	glBegin(GL_LINE_STRIP);
		glVertex3f(corners[0].x, corners[0].y, corners[0].z);
		glVertex3f(corners[1].x, corners[1].y, corners[1].z);
		glVertex3f(corners[5].x, corners[5].y, corners[5].z);
		glVertex3f(corners[4].x, corners[4].y, corners[4].z);
		glVertex3f(corners[0].x, corners[0].y, corners[0].z);
		glVertex3f(corners[3].x, corners[3].y, corners[3].z);
		glVertex3f(corners[7].x, corners[7].y, corners[7].z);
		glVertex3f(corners[6].x, corners[6].y, corners[6].z);
		glVertex3f(corners[2].x, corners[2].y, corners[2].z);
		glVertex3f(corners[1].x, corners[1].y, corners[1].z);
	glEnd();
	glBegin(GL_LINES);
		glVertex3f(corners[3].x, corners[3].y, corners[3].z);
		glVertex3f(corners[2].x, corners[2].y, corners[2].z);
		glVertex3f(corners[7].x, corners[7].y, corners[7].z);
		glVertex3f(corners[4].x, corners[4].y, corners[4].z);
		glVertex3f(corners[6].x, corners[6].y, corners[6].z);
		glVertex3f(corners[5].x, corners[5].y, corners[5].z);
	glEnd();
	glPopMatrix();
	return true;
}

int CRenderWOGL::registerTexture(int width, int height, const void* data)
{
	int id = -1;
	id = createOGLTexture(width, height, data);
	return id;
}

bool CRenderWOGL::setTextureActive(int id)
{
	glBindTexture(GL_TEXTURE_2D, id);
	return true;
}

bool CRenderWOGL::setProjectionMatrix(float* matrix)
{
/*
 	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(matrix);
	glMatrixMode(GL_MODELVIEW);
*/
	return true;
}

bool CRenderWOGL::setViewport(int x, int y, int width, int height)
{
	glViewport(x, y, width, height);
	return true;
}

CRenderWOGL::CRenderWOGL(): useGLUT(false),
														hwnd(NULL),
														hdc(NULL),
														context(NULL)
{
	typeName = "CRenderWOGL";
}

CRenderWOGL::~CRenderWOGL()
{
	end();
}

GLenum CRenderWOGL::E3D_MODE_TO_GL_MODE(E3D_MODE mode)
{
	GLenum glMode;
	switch(mode)
	{
		case E3D_POINTS:										glMode = GL_POINTS;											break;
		case E3D_LINES:											glMode = GL_LINES;											break;
		case E3D_LINE_LOOP:									glMode = GL_LINE_LOOP;									break;
		case E3D_LINE_STRIP:								glMode = GL_LINE_STRIP;									break;
		case E3D_TRIANGLES:									glMode = GL_TRIANGLES;									break;
		case E3D_TRIANGLE_STRIP:						glMode = GL_TRIANGLE_STRIP;							break;
		case E3D_TRIANGLE_FAN:							glMode = GL_TRIANGLE_FAN;								break;
		case E3D_QUADS:											glMode = GL_QUADS;											break;
		case E3D_QUAD_STRIP:								glMode = GL_QUAD_STRIP;									break;
		case E3D_POLYGON:										glMode = GL_POLYGON;										break;
	}
	return glMode;
}

GLuint CRenderWOGL::createOGLTexture(GLint width, GLint height, const GLvoid* data)
{
	GLuint id;

	glGenTextures(1, &id);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glBindTexture(GL_TEXTURE_2D, id);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	// GL_MODULATE
	// GL_DECAL
  // GL_BLEND
  // GL_REPLACE
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	// GLenum target
	// GLint level
	// GLint internalformat
	// GLsizei width
	// GLsizei height
	// GLint border
	// GLenum format
	// GLenum type
	// const GLvoid *pixels

/*
#define GL_RGB16                          0x8054
#define GL_RGBA2                          0x8055
#define GL_RGBA4                          0x8056
#define GL_RGB5_A1                        0x8057
#define GL_RGBA8                          0x8058
#define GL_RGB10_A2                       0x8059
#define GL_RGBA12                         0x805A
#define GL_RGBA16                         0x805B
*/
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB16, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data);

	return id;
}


