/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CRender_h_
#define _E3D_CRender_h_

#include "E3D_CObject.h"

typedef unsigned int E3Denum;

enum E3D_MODE
{
	E3D_POINTS,
	E3D_LINES,
	E3D_LINE_LOOP,
	E3D_LINE_STRIP,
	E3D_TRIANGLES,
	E3D_TRIANGLE_STRIP,
	E3D_TRIANGLE_FAN,
	E3D_QUADS,
	E3D_QUAD_STRIP,
	E3D_POLYGON
};

enum E3D_DRAW_BUFFER_MODE
{
	E3D_FRONT,
	E3D_BACK,
	E3D_FRONT_AND_BACK
};

enum E3D_TARGET
{
	E3D_TEXTURE_2D
};

class CIndex;

class CRenderModel: public CObject
{
public:
	CRenderModel(): numIndices(0),
									numVertices(0),
									mode(E3D_POINTS),
									indices(NULL),
									positions(NULL),
									normals(NULL),
									uvCoords(NULL),
									colors(NULL)
	{
	}
									
	int numIndices;
	int numVertices;
	E3D_MODE mode;
	E3D_DRAW_BUFFER_MODE drawBufferMode;
	CIndex* indices;
	float* positions;
	float* normals;
	float* uvCoords;
	float* colors;
};

class CBoundingBox;

class CRender: public CObject
{
public:
	// Own
	CRender();
	virtual ~CRender();

	// Has to be implemented inside inherited classes
	virtual bool init(void* window, int width, int height, bool fullscreen) = 0;
	virtual bool end() = 0;
	virtual bool startFrame() = 0;
	virtual bool flip() = 0;
	virtual bool endFrame() = 0;
	virtual bool pushModelMatrix(float* modelMatrix) = 0;
	virtual bool popModelMatrix() = 0;
	virtual bool renderModel() = 0;
	virtual bool setViewMatrix(float* viewMatrix) = 0;
	virtual bool renderBoundingBox(const CBoundingBox& bd) = 0;
	virtual int registerTexture(int width, int height, const void* data) = 0;
	virtual bool setTextureActive(int id) = 0;
	virtual bool setProjectionMatrix(float* matrix) = 0;
	virtual bool setViewport(int x, int y, int width, int height) = 0;

	// Allready has an implementation
	virtual bool setModelMode(E3D_MODE mode);
	virtual bool setModelIndices(CIndex* indices);
	virtual bool setModelNumIndices(int ni);
	virtual bool setModelNumVertices(int nv);
	virtual bool setModelPositions(float* positions);
	virtual bool setModelNormals(float* normals);
	virtual bool setModelUVCoords(float* uvCoords);
	virtual bool setModelColors(float* colors);

protected:
	CRenderModel model;
};

#endif