/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CAnimationCurve.h"

#include <algorithm>
using std::find;

DEFINE_CREATOR_OBJECT(CAnimationCurve)

bool CKey::operator==(const CKey& key)
{
	return time == key.time;
}

bool CKey::read(IReader* reader)
{
	if (reader == NULL) return false;
	bool result = true;
	if (result) result = reader->read("Time", time);
	if (result) result = reader->read("Value", value);
	return result;
}

bool CKey::write(IWriter* writer)
{
	if (writer == NULL) return false;
	bool result = true;
	if (result) result = writer->write("Time", time);
	if (result) result = writer->write("Value", value);
	return result;
}

void CAnimationCurve::update(float time)
{
//	dirty |= updateConnections(time);

	lastTime = currentTime;
	currentTime = time;

	// If there are no keys... return
	if (keys.size() == 0) // No keys
	{
		return;
	}
	else if (keys.size() == 1 || time < keys.front().time) // Just one key or time above any of them
	{
		actualValue = keys.front().value;
	}
	else if (time > keys.back().time) // Time below all the keys
	{
		actualValue = keys.back().value;
	}
	else // Time between two keys... find them!
	{
		CKey previous, next;
		previous = next = keys[0];
		bool found = false;
		for (int i=1; !found && i<keys.size(); i++)
		{
			previous = next;
			next = keys[i];
			found = time < next.time;
		}
		float dif = next.time - previous.time;
		float t = (time - previous.time) / dif;
		actualValue = (1 - t) * previous.value + t * next.value;
	}

	dirty = false;
}

float* CAnimationCurve::getAttribute(const string& attributeName)
{
	float* attrib = NULL;
	if ((attrib = CBase::getAttribute(attributeName)) == NULL)
	{
		if (attributeName == "output")
		{
			attrib = &actualValue;
		}
	}
	return attrib;
}

bool CAnimationCurve::read(IReader* reader)
{
	if (reader == NULL) return false;
	bool result = CBase::read(reader);
	if (result) result = reader->push("CAnimationCurve");
	if (result)
	{
		keys.clear();
		int numKeys;
		result = reader->read("Num", numKeys);
		for (int i=0; i < numKeys && result; i++)
		{
			CKey key;
			result = key.read(reader);
			if (result) result = addKey(key);
		}
	}
	if (result) result = reader->pop("CAnimationCurve");
	return result;
}

bool CAnimationCurve::write(IWriter* writer)
{
	if (writer == NULL) return false;
	bool result = CBase::write(writer);
	if (result) result = writer->push("CAnimationCurve");
	if (result)
	{
		result = writer->write("Num", (int)keys.size());
		CKeyVectorIterator it = keys.begin();
		for (; it != keys.end() && result; it++)
		{
			result = it->write(writer);
		}
	}
	if (result) result = writer->pop("CAnimationCurve");
	return result;
}

CAnimationCurve::CAnimationCurve()
{
	typeName = "CAnimationCurve";
}

bool CAnimationCurve::addKey(const CKey& key)
{
	bool result = false;
	CKeyVectorIterator it = find(keys.begin(), keys.end(), key);
	if (it == keys.end())
	{
		keys.push_back(key);
		result = true;
	}
	return result;
}