/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CCamera.h"
#include "E3D_CRender.h"
#include "E3D_IReader.h"
#include "E3D_IWriter.h"

DEFINE_CREATOR_OBJECT(CCamera)

bool CCamera::read(IReader* reader)
{
	if (reader == NULL) return false;
	bool result = CHierarchyBase::read(reader);
	if (result) result = reader->push("CCamera");
	if (result) result = reader->read("Left", left);
	if (result) result = reader->read("Right", right);
	if (result) result = reader->read("Bottom", bottom);
	if (result) result = reader->read("Top", top);
	if (result) result = reader->read("ZNear", zNear);
	if (result) result = reader->read("ZFar", zFar);
	if (result) result = reader->pop("CCamera");
	return result;
}

bool CCamera::write(IWriter* writer)
{
	if (writer == NULL) return false;
	bool result = CHierarchyBase::write(writer);
	if (result) result = writer->push("CCamera");
	if (result) result = writer->write("Left", left);
	if (result) result = writer->write("Right", right);
	if (result) result = writer->write("Bottom", bottom);
	if (result) result = writer->write("Top", top);
	if (result) result = writer->write("ZNear", zNear);
	if (result) result = writer->write("ZFar", zFar);
	if (result) result = writer->pop("CCamera");
	return result;
}

void CCamera::update(float time)
{
	if (dirty)
	{
		projectionMatrix.perspectiveProjection(left, right, bottom, top, zNear, zFar);
		projectionMatrix.toArray(projectionMatrixArray);
	}
}

bool CCamera::updateHierarchy(CMatrix4D& parentWorldMatrix, CBoundingBox& parentBoundingBox, bool parentDirty)
{
	bool updated = dirty | parentDirty;
	if (updated)
	{
		worldMatrix = CMatrix4D::inverse(parentWorldMatrix);
		worldMatrix.toArray(worldMatrixArray);
	}
	dirty = false;
	return updated;
}

CCamera::CCamera(): left(0), right(0), bottom(0), top(0), zNear(0), zFar(0)
{
	typeName = "CCamera";
}

void CCamera::setupToRender(CRender* render)
{
	if (render == NULL) return;
	render->setProjectionMatrix(projectionMatrixArray);
	render->setViewMatrix(worldMatrixArray);
}

void CCamera::setFrustum(float l, float r, float b, float t, float zn, float zf)
{
	left = l;
	right = r;
	bottom = b;
	top = t;
	zNear = zn;
	zFar = zf;
	dirty = true;
}

void CCamera::calcFrustum()
{
}

