/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CConnection.h"
#include "E3D_IWriter.h"
#include "E3D_IReader.h"

using std::bad_cast;

CConnection::CConnection():	srcNode(NULL),
														srcAttribute(NULL),
														dstNode(NULL),
														dstAttribute(NULL)
{
}

bool CConnection::operator==(const CObject& obj) const
{
	bool result = false;
	try
	{
		const CConnection& connection = static_cast<const CConnection&>(obj);
		result =	(srcNode == connection.srcNode) && 
							(srcNodeName == connection.srcNodeName) &&
							(srcAttribute == connection.srcAttribute) &&
							(srcAttributeName == connection.srcAttributeName) &&
							(dstNode == connection.dstNode) && 
							(dstNodeName == connection.dstNodeName) &&
							(dstAttribute == connection.dstAttribute) &&
							(dstAttributeName == connection.dstAttributeName);
	}
	catch(bad_cast e)
	{
	}
	return result;
}

CConnection::operator string() const
{
	char buf[256];
	sprintf(buf, "%s.%s = %s.%s", dstNodeName.c_str(), dstAttributeName.c_str(), srcNodeName.c_str(), srcAttributeName.c_str());
	return buf;
}

bool CConnection::read(IReader* reader)
{
	if (reader == NULL) return false;
	bool result = true;
	if (result) result = reader->read("SrcNodeName", srcNodeName);
	if (result) result = reader->read("SrcAttributeName", srcAttributeName);
	if (result) result = reader->read("DstNodeName", dstNodeName);
	if (result) result = reader->read("DstAttributeName", dstAttributeName);
	return result;
}

bool CConnection::write(IWriter* writer)
{
	if (writer == NULL) return false;
	bool result = true;
	if (result) result = writer->write("SrcNodeName", srcNodeName);
	if (result) result = writer->write("SrcAttributeName", srcAttributeName);
	if (result) result = writer->write("DstNodeName", dstNodeName);
	if (result) result = writer->write("DstAttributeName", dstAttributeName);
	return result;
}

bool CConnection::isConnected()
{
	return	srcNode != NULL && srcAttribute != NULL && 
					dstNode != NULL && dstAttribute != NULL;
}
