/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CCAMERA_H_
#define _E3D_CCAMERA_H_

#include "E3D_CHierarchyBase.h"

class IReader;
class IWriter;

class CCamera: public CHierarchyBase
{
public:
// From CBase
	// ============================================================================
	// Method to read all the data related with this class from a reader interface.
	// ============================================================================
	virtual bool read(IReader* reader);

	// ============================================================================
	// Method to write all the data related with this class to a writer interface
	// ============================================================================
	virtual bool write(IWriter* writer);

	// ============================================================================
	// Updates any object. Polimorph it to give it your own behaivour.
	// In this case just updates the timing.
	// ============================================================================
	virtual void update(float time);

// From CHierarchyBase
	// ============================================================================
	// In this case the parent matrix is the world matrix of the camera but it needs an
	// inversion in order to concatenate it to each model and transform from world
	// matrix to view matrix.
	// ============================================================================
	virtual bool updateHierarchy(CMatrix4D& parentWorldMatrix, CBoundingBox& parentBoundingBox, bool parentDirty);

// Own
	// ============================================================================
	// ============================================================================
	CCamera();

	// ============================================================================
	// ============================================================================
	void setupToRender(CRender* render);

	// ============================================================================
	// ============================================================================
	void setFrustum(float l, float r, float b, float t, float zn, float zf);

protected:
	void calcFrustum();
	float horizontalFOV, verticalFOV, aspectRatio, left, right, bottom, top, zNear, zFar;
	CMatrix4D projectionMatrix;
	float projectionMatrixArray[16];
};

DEFINE_CREATOR_CLASS(CCamera)

#endif