/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CGRAPH_H_
#define _E3D_CGRAPH_H_

#include "E3D_CHierarchyBase.h"

class IReader;
class IWriter;

class CGraph: public CHierarchyBase
{
public:
// From CBase
	// ============================================================================
	// Method to read all the data related with this class from a reader interface.
	// ============================================================================
	virtual bool read(IReader* reader);

	// ============================================================================
	// Method to write all the data related with this class to a writer interface
	// ============================================================================
	virtual bool write(IWriter* writer);

	// ============================================================================
	// Updates any object. Polimorph it to give it your own behaivour.
	// In this case just updates the timing.
	// 
	// ============================================================================
	virtual void update(float time);

// Own
	// ============================================================================
	// ============================================================================
	CGraph();

	// ============================================================================
	// ============================================================================
	~CGraph();

	// ============================================================================
	// Add a node to the scene. Return false either if the node is NULL or the node
	// name allready exists in the scene.
	// ============================================================================
	bool addNode(CBase* node);

	// ============================================================================
	// Get a node by its name. Returns NULL if a node with that name cannot be found.
	// ============================================================================
	CBase* getNode(const string& name);

	// ============================================================================
	// ============================================================================
	void setPause(bool pause);

	// ============================================================================
	// ============================================================================
	bool getPause();

	// ============================================================================
	// ============================================================================
	void end();

	// ============================================================================
	// ============================================================================
	bool connectAll();


protected:
	bool writeHierarchy(IWriter* writer, CHierarchyBase* node);
	void getHierarchy(CHierarchyBase* parent, pCHierarchyBaseVector& hierarchy);
	void getConnections(pCConnectionVector& connections);

	bool pause;
	stringPCBaseMap nodes;
};

#endif