/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CUVCoord_h_
#define _E3D_CUVCoord_h_

#include "E3D_CObject.h"

#include <vector>
using std::vector;

// ============================================================================
// The class to store the data for 3D texturing. This UV coordinates refers to
// every viertex that want to be textured.
// ============================================================================
class CUVCoord: public CObject
{
public:
// From CObject
	// ============================================================================
	// An operator to cast any object to a string. Polimorph it to give it a new
	// behaviour that fits your needs.
	// ============================================================================
	virtual operator string() const;

	// ============================================================================
	// An operator to compare the equality of two objects. Polimorph it to give
	// a new behaviour that fits your needs.
	// ============================================================================
	virtual bool operator==(const CObject& obj) const;

	// ============================================================================
	// A function to take all the data that an object needs from an string.
	// ============================================================================
	virtual bool fromString(const string& from);

// Own
	// ============================================================================
	// The constructor to initialize some data by default.
	// ============================================================================
	CUVCoord();

	// ============================================================================
	// The constructor to give some values by default.
	// ============================================================================
	CUVCoord(float u, float v);

	float u, v;
};

typedef vector<CUVCoord> CUVCoordVector;
typedef CUVCoordVector::iterator CUVCoordVectorIterator;


#endif