/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CApplication.h"

CApplication::CApplication()
{
	typeName = "CApplication";
}

bool CApplication::eventResize(int width, int height)
{
	return true;
}

bool CApplication::eventClose()
{
	return false;
}

//#define USE_GLUT
#define _CONSOLE

CApplication* application = NULL;

#ifdef USE_GLUT
// =============================================================================
// GLUT
// =============================================================================
	#include <glut.h>

	int windowID;

	void glutDisplay()
	{
		if (application != NULL)
		{
			if (!application->update())
			{
				glutDestroyWindow(windowID);
			}
		}
	}

	void glutIdle()
	{
		glutDisplay();
	}

	void glutReshape(int width, int height)
	{
		if (application != NULL)
		{
			application->eventResize(width, height);
		}
	}

	#ifdef _WINDOWS
	#include <windows.h>
	int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hprevinstance, LPSTR cmdline, int nshowcmd)
	{
		int argc = 0;
		char** argv = {{NULL}};
	#else
	int main(int argc, char** argv)
	{
	#endif
		glutInit(&argc, argv);
		glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
		glutInitWindowSize(0, 0);
		glutInitWindowPosition(0, 0);
		windowID = glutCreateWindow("Engine3D v0.1");
		glutIdleFunc(glutIdle);
		glutDisplayFunc(glutDisplay);
		glutReshapeFunc(glutReshape);
		application = CApplication::create();
		if (application != NULL)
		{
			application->window = NULL;
			if (application->init())
			{
				glutMainLoop();
				application->end();
			}
		}
		delete application;
		return 0;
	}
#else
// =============================================================================
// WIN32 API
// =============================================================================
	#ifdef WIN32
		#include <windows.h>
		
		LRESULT CALLBACK windowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
		{
			switch (msg)
			{
				case WM_SIZE:
				{
 					int width = LOWORD(lparam);
					int height = HIWORD(lparam);
					if (application != NULL)
					{
						application->eventResize(width, height);
					}
					return 0;
				}
				case WM_DESTROY:
					if (application != NULL)
					{
						if (!application->eventClose())
						{
							PostQuitMessage(0);
						}
					}
					return 0;
			}
			return DefWindowProc(hwnd, msg, wparam, lparam);
		}

		void* createWindow(HINSTANCE hinstance, int nshowcmd)
		{
			static char name[] = "Engine3D";
			HWND hwnd;
			WNDCLASSEX wndClassEx;

			wndClassEx.cbSize = sizeof(wndClassEx);
			wndClassEx.style = CS_HREDRAW | CS_VREDRAW;
			wndClassEx.lpfnWndProc = windowProc;
			wndClassEx.cbClsExtra = 0;
			wndClassEx.cbWndExtra = 0;
			wndClassEx.hInstance = hinstance;
			wndClassEx.hCursor = LoadCursor(NULL, IDC_ARROW);
			wndClassEx.hIcon = LoadIcon(NULL, IDI_APPLICATION);
			wndClassEx.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
			wndClassEx.lpszMenuName = NULL;
			wndClassEx.lpszClassName = name;
			wndClassEx.hIconSm = LoadIcon(NULL, IDI_APPLICATION);

			RegisterClassEx(&wndClassEx);

			hwnd = CreateWindow(name, "Engine3D v0.1", WS_OVERLAPPEDWINDOW, 
													CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
													NULL, NULL, hinstance, NULL);

			ShowWindow(hwnd, nshowcmd);
			UpdateWindow(hwnd);

			return (void*)hwnd;
		}

		#ifdef _CONSOLE
// =============================================================================
		// CONSOLE
// =============================================================================
			int main(int argc, char** argv)
			{
				int nshowcmd = 1;
				HINSTANCE hinstance = NULL;
		#else
// =============================================================================
		// WINDOW
// =============================================================================
			int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hprevinstance, LPSTR cmdline, int nshowcmd)
			{
		#endif
			application = CApplication::create();
			if (application != NULL)
			{
				application->window = createWindow(hinstance, nshowcmd);
				if (application->init())
				{
					bool endApplication = false;
					MSG msg;
					do 
					{
						if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
						{
							if (msg.message == WM_PAINT)
							{
								endApplication = !application->update();
							}
							TranslateMessage(&msg);
							DispatchMessage(&msg);
						}
						else
						{
							endApplication = !application->update();
						}
					} while (!endApplication && msg.message != WM_QUIT);
					application->end();
				}
				delete application;
				PostQuitMessage(0);
			}
			return 0;
		}
	#endif
#endif

