/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CInput.h"

CInputEvent::CInputEvent(CInputData* id): inputData(id)
{
}

CInputData* CInputEvent::getInputData()
{
	return inputData;
}

CInput::CInput(): inputData(false)
{
	typeName = "CInput";
}

CInput::~CInput()
{
	end();
}

void CInput::update(float time)
{
	bool launchEvent = false;
	stringPCInputDeviceMapIterator it = devices.begin();
	for (; it != devices.end(); it++)
	{
		it->second->update(time);
		launchEvent |= it->second->getChanged();
	}
	if (launchEvent)
	{
		CInputEvent ie(&inputData);
		pIInputListenerVectorIterator it = inputListeners.begin();
		for (; it != inputListeners.end(); it++)
		{
			(*it)->inputChanged(ie);
		}
	}
}

bool CInput::addInputDevice(CInputDevice* device)
{
	bool result = false;
	if (device != NULL)
	{
		result = devices.insert(stringPCInputDeviceMap::value_type(device->getName(), device)).second;
		if (result)
		{
			inputData.add(device->getInputData());
		}
	}
	return result;
}

CInputButton* CInput::getInputButton(const string& name)
{
	return inputData.getInputButton(name);
}

CInputCursor* CInput::getInputCursor(const string& name)
{
	return inputData.getInputCursor(name);
}

void CInput::addInputListener(IInputListener* il)
{
	if (il == NULL) return;
	inputListeners.push_back(il);
}

void CInput::end()
{
	stringPCInputDeviceMapIterator it = devices.begin();
	for (; it != devices.end(); it++)
	{
		delete it->second;
		it->second = NULL;
	}
	inputData.end();
	clear();
}

void CInput::clear()
{
	devices.clear();
	inputData.clear();
}