/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CInputData.h"

CInputData::CInputData(bool deleteAtEnd): deleteAtEnd(deleteAtEnd)
{
	typeName = "CInputData";
}

CInputData::~CInputData()
{
	end();
}

bool CInputData::addInputButton(CInputButton* button)
{
	bool result = false;
	if (button != NULL)
	{
		result = buttons.insert(stringPCInputButtonMap::value_type(button->getName(), button)).second;
	}
	return result;
}

bool CInputData::addInputCursor(CInputCursor* cursor)
{
	bool result = false;
	if (cursor != NULL)
	{
		result = cursors.insert(stringPCInputCursorMap::value_type(cursor->getName(), cursor)).second;
	}
	return result;
}

void CInputData::add(CInputData* iData)
{
	if (iData == NULL) return;
	stringPCInputButtonMapIterator bIT;
	stringPCInputCursorMapIterator cIT;
	bIT = iData->buttons.begin();
	for (; bIT != iData->buttons.end() && addInputButton(bIT->second); bIT++);
	cIT = iData->cursors.begin();
	for (; cIT != iData->cursors.end() && addInputCursor(cIT->second); bIT++);
}

CInputButton* CInputData::getInputButton(const string& name)
{
	CInputButton* button = NULL;
	stringPCInputButtonMapIterator bIT;
	bIT = buttons.find(name);
	if (bIT != buttons.end())
	{
		button = bIT->second;
	}
	return button;
}

CInputCursor* CInputData::getInputCursor(const string& name)
{
	CInputCursor* cursor = NULL;
	stringPCInputCursorMapIterator cIT;
	cIT = cursors.find(name);
	if (cIT != cursors.end())
	{
		cursor = cIT->second;
	}
	return cursor;
}

void CInputData::end()
{
	if (deleteAtEnd)
	{
		stringPCInputButtonMapIterator bIT;
		stringPCInputCursorMapIterator cIT;
		bIT = buttons.begin();
		for (; bIT != buttons.end(); bIT++)
		{
			delete bIT->second;
			bIT->second = NULL;
		}
		cIT = cursors.begin();
		for (; cIT != cursors.end(); bIT++)
		{
			delete cIT->second;
			cIT->second = NULL;
		}
	}
	clear();
}

void CInputData::clear()
{
	buttons.clear();
	cursors.clear();
}
