/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CInputDeviceKeyboardW32.h"

#include <windows.h>

#include <iostream>
using std::cout;
using std::endl;

void CInputDeviceKeyboardW32::update(float time)
{
	if (!enabled)	return;

	// Start thinking that the device is not going to change
	changed = false;

	BUTTON_CODE_W32 W32ButtonCode;
	string buttonName;
	stringPCInputButtonMapIterator buttonIT;
	CInputButton* button = NULL;

	// Check out every button inside the W32 to generic button map
	for (BUTTON_CODE_W32StringMapIterator it = W32ButtonMap.begin(); it != W32ButtonMap.end(); it++)
	{
		W32ButtonCode = it->first;
		buttonName = it->second;
		if (W32ButtonCode != BUTTON_CODE_W32_NOBUTTON)
		{
			// Get the correct button
			button = inputData.getInputButton(buttonName);
			if (button != NULL)
			{
				// Get the state of a key and set it to the button
				button->setPressed(GetAsyncKeyState(W32ButtonCode) != 0);
				// Only make the update if pressed or changed
				changed |= button->getPressed() || button->getChanged();
			}
		}
	}
}

CInputDeviceKeyboardW32::CInputDeviceKeyboardW32()
{
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NOBUTTON						, CInputButton::BUTTON_NAME_NOBUTTON					));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_BACK_SPACE					, CInputButton::BUTTON_NAME_BACK_SPACE				));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_TAB								, CInputButton::BUTTON_NAME_TAB								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_RETURN							, CInputButton::BUTTON_NAME_RETURN						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_INTRO							, CInputButton::BUTTON_NAME_INTRO							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_ESCAPE							, CInputButton::BUTTON_NAME_ESCAPE						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_CAPS_LOCK					, CInputButton::BUTTON_NAME_CAPS_LOCK					));

	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_SHIFT_RIGHT				, CInputButton::BUTTON_NAME_SHIFT_RIGHT				));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_SHIFT_LEFT					, CInputButton::BUTTON_NAME_SHIFT_LEFT				));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_CONTROL_RIGHT			, CInputButton::BUTTON_NAME_CONTROL_RIGHT			));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_CONTROL_LEFT				, CInputButton::BUTTON_NAME_CONTROL_LEFT			));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_ALT_RIGHT					, CInputButton::BUTTON_NAME_ALT_RIGHT					));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_ALT_LEFT						, CInputButton::BUTTON_NAME_ALT_LEFT					));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_MENU								, CInputButton::BUTTON_NAME_MENU							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_LWIN								, CInputButton::BUTTON_NAME_LWIN							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_RWIN								, CInputButton::BUTTON_NAME_RWIN							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NN									, CInputButton::BUTTON_NAME_NN								));
                                             
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_OPEN_BRANCH				, CInputButton::BUTTON_NAME_OPEN_BRANCH				));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_CLOSE_BRANCH				, CInputButton::BUTTON_NAME_CLOSE_BRANCH			));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_ARROWS							, CInputButton::BUTTON_NAME_ARROWS						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_COMMA							, CInputButton::BUTTON_NAME_COMMA							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_MINUS							, CInputButton::BUTTON_NAME_MINUS							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_POINT							, CInputButton::BUTTON_NAME_POINT							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_SLASH							, CInputButton::BUTTON_NAME_SLASH							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_BACK_QUOTE					, CInputButton::BUTTON_NAME_BACK_QUOTE				));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_OPEN_BRACKET				, CInputButton::BUTTON_NAME_OPEN_BRACKET			));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_CLOSE_BRACKET			, CInputButton::BUTTON_NAME_CLOSE_BRACKET			));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_QUOTE							, CInputButton::BUTTON_NAME_QUOTE							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_SPACE							, CInputButton::BUTTON_NAME_SPACE							));

	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_PAGE_UP						, CInputButton::BUTTON_NAME_PAGE_UP						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_PAGE_DOWN					, CInputButton::BUTTON_NAME_PAGE_DOWN					));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_END								, CInputButton::BUTTON_NAME_END								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_HOME								, CInputButton::BUTTON_NAME_HOME							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_LEFT								, CInputButton::BUTTON_NAME_LEFT							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_UP									, CInputButton::BUTTON_NAME_UP								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_RIGHT							, CInputButton::BUTTON_NAME_RIGHT							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_DOWN								, CInputButton::BUTTON_NAME_DOWN							));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_INSERT							, CInputButton::BUTTON_NAME_INSERT						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_DELETE							, CInputButton::BUTTON_NAME_DELETE						));
	
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_0									, CInputButton::BUTTON_NAME_0									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_1									, CInputButton::BUTTON_NAME_1									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_2									, CInputButton::BUTTON_NAME_2									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_3									, CInputButton::BUTTON_NAME_3									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_4									, CInputButton::BUTTON_NAME_4									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_5									, CInputButton::BUTTON_NAME_5									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_6									, CInputButton::BUTTON_NAME_6									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_7									, CInputButton::BUTTON_NAME_7									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_8									, CInputButton::BUTTON_NAME_8									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_9									, CInputButton::BUTTON_NAME_9									));

	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_A									, CInputButton::BUTTON_NAME_A									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_B									, CInputButton::BUTTON_NAME_B									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_C									, CInputButton::BUTTON_NAME_C									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_D									, CInputButton::BUTTON_NAME_D									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_E									, CInputButton::BUTTON_NAME_E									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F									, CInputButton::BUTTON_NAME_F									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_G									, CInputButton::BUTTON_NAME_G									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_H									, CInputButton::BUTTON_NAME_H									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_I									, CInputButton::BUTTON_NAME_I									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_J									, CInputButton::BUTTON_NAME_J									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_K									, CInputButton::BUTTON_NAME_K									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_L									, CInputButton::BUTTON_NAME_L									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_M									, CInputButton::BUTTON_NAME_M									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_N									, CInputButton::BUTTON_NAME_N									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_O									, CInputButton::BUTTON_NAME_O									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_P									, CInputButton::BUTTON_NAME_P									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_Q									, CInputButton::BUTTON_NAME_Q									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_R									, CInputButton::BUTTON_NAME_R									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_S									, CInputButton::BUTTON_NAME_S									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_T									, CInputButton::BUTTON_NAME_T									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_U									, CInputButton::BUTTON_NAME_U									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_V									, CInputButton::BUTTON_NAME_V									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_W									, CInputButton::BUTTON_NAME_W									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_X									, CInputButton::BUTTON_NAME_X									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_Y									, CInputButton::BUTTON_NAME_Y									));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_Z									, CInputButton::BUTTON_NAME_Z									));

	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMLOCK						, CInputButton::BUTTON_NAME_NUMLOCK						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD0						, CInputButton::BUTTON_NAME_NUMPAD0						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD1						, CInputButton::BUTTON_NAME_NUMPAD1						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD2						, CInputButton::BUTTON_NAME_NUMPAD2						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD3						, CInputButton::BUTTON_NAME_NUMPAD3						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD4						, CInputButton::BUTTON_NAME_NUMPAD4						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD5						, CInputButton::BUTTON_NAME_NUMPAD5						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD6						, CInputButton::BUTTON_NAME_NUMPAD6						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD7						, CInputButton::BUTTON_NAME_NUMPAD7						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD8						, CInputButton::BUTTON_NAME_NUMPAD8						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_NUMPAD9						, CInputButton::BUTTON_NAME_NUMPAD9						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_MULTIPLY						, CInputButton::BUTTON_NAME_MULTIPLY					));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_ADD								, CInputButton::BUTTON_NAME_ADD								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_SUBSTRACT					, CInputButton::BUTTON_NAME_SUBSTRACT					));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_DECIMAL						, CInputButton::BUTTON_NAME_DECIMAL						));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_DIVIDE							, CInputButton::BUTTON_NAME_DIVIDE						));

	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F1									, CInputButton::BUTTON_NAME_F1								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F2									, CInputButton::BUTTON_NAME_F2								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F3									, CInputButton::BUTTON_NAME_F3								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F4									, CInputButton::BUTTON_NAME_F4								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F5									, CInputButton::BUTTON_NAME_F5								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F6									, CInputButton::BUTTON_NAME_F6								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F7									, CInputButton::BUTTON_NAME_F7								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F8									, CInputButton::BUTTON_NAME_F8								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F9									, CInputButton::BUTTON_NAME_F9								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F10								, CInputButton::BUTTON_NAME_F10								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F11								, CInputButton::BUTTON_NAME_F11								));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_F12								, CInputButton::BUTTON_NAME_F12								));

	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_PRINTSCREEN				, CInputButton::BUTTON_NAME_PRINTSCREEN				));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_SCROLL_LOCK				, CInputButton::BUTTON_NAME_SCROLL_LOCK				));
	W32ButtonMap.insert(BUTTON_CODE_W32StringMap::value_type(BUTTON_CODE_W32_PAUSE							, CInputButton::BUTTON_NAME_PAUSE							));

	CInputButton* button;
	for (BUTTON_CODE_W32StringMapIterator it = W32ButtonMap.begin(); it != W32ButtonMap.end(); it++)
	{
		button = new CInputButton();
		button->setName(it->second);
		inputData.addInputButton(button);
	}
	name = DEVICE_NAME_KEYBOARD;
}