/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CInputDeviceMouseW32.h"

#include <windows.h>

void CInputDeviceMouseW32::update(float time)
{
	if (!enabled) return;

	// Start thinking that the device is not going to change
	changed = false;

	CInputCursor* cursor = inputData.getInputCursor(CInputCursor::CURSOR_NAME_MOUSE);

	// Update mouse cursor 
	if (cursor != NULL)
	{
		float lastX = cursor->getPosX();
		float lastY = cursor->getPosY();
		POINT point;
		if (GetCursorPos(&point))
		{
			cursor->setPosX(point.x);
			cursor->setPosY(point.y);
			cursor->setPosZ(0.0f);

			if (lastX != point.x || lastY != point.y)
			{
				cursor->setChanged(true);
			}
			else
			{
				cursor->setChanged(false);
			}
			changed |= cursor->getChanged();

			cursor->setIncX(point.x - lastX);
			cursor->setIncY(point.y - lastY);
			cursor->setIncZ(0.0f);
		}
	}

	// Update mouse buttons
	CInputButton* button = NULL;
	button = inputData.getInputButton(CInputButton::BUTTON_NAME_MOUSE_LEFT);
	if (button != NULL)
	{
		button->setPressed(GetAsyncKeyState(VK_LBUTTON) != 0);
		changed |= button->getPressed() || button->getChanged();
	}
	button = inputData.getInputButton(CInputButton::BUTTON_NAME_MOUSE_RIGHT);
	if (button != NULL)
	{
		button->setPressed(GetAsyncKeyState(VK_RBUTTON) != 0);
		changed |= button->getPressed() || button->getChanged();
	}
	button = inputData.getInputButton(CInputButton::BUTTON_NAME_MOUSE_MIDDLE);
	if (button != NULL)
	{
		button->setPressed(GetAsyncKeyState(VK_MBUTTON) != 0);
		changed |= button->getPressed() || button->getChanged();
	}
}

CInputDeviceMouseW32::CInputDeviceMouseW32()
{
	// Insert mouse buttons
	CInputButton* bMR = new CInputButton();
	bMR->setName(CInputButton::BUTTON_NAME_MOUSE_RIGHT);
	inputData.addInputButton(bMR);

	CInputButton* bML = new CInputButton();
	bML->setName(CInputButton::BUTTON_NAME_MOUSE_LEFT);
	inputData.addInputButton(bML);

	CInputButton* bMM = new CInputButton();
	bMM->setName(CInputButton::BUTTON_NAME_MOUSE_MIDDLE);
	inputData.addInputButton(bMM);

	// Insert mouse cursor
	CInputCursor* cM = new CInputCursor();
	cM->setName(CInputCursor::CURSOR_NAME_MOUSE);
	inputData.addInputCursor(cM);

	name = DEVICE_NAME_MOUSE;
}
