/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CReaderASCII.h"
#include "E3D_CVector3D.h"
#include "E3D_CColor.h"
#include "E3D_CUVCoord.h"

#include <sstream>
using std::stringstream;

bool CReaderASCII::open(const string& file)
{
	bool result = true;
	if (result = close())
	{
		ifs.open(file.c_str());
		result = ifs.is_open();
	}
	return result;
}

bool CReaderASCII::close()
{
	bool result = true;
	if (ifs.is_open())
	{
		ifs.close();
	}
	return result;
}

bool CReaderASCII::push(const string& name)
{
	if (!ifs.is_open()) return false;
	string line;
	bool result = findLine("begin " + name, line);
	return result;
}

bool CReaderASCII::pop(const string& name)
{
	if (!ifs.is_open()) return false;
	string line;
	bool result = findLine("end " + name, line);
	return result;
}

bool CReaderASCII::read(const string& name)
{
	if (!ifs.is_open()) return false;
	bool result = true;
	return result;
}

bool CReaderASCII::read(const string& name, float& value)
{
	if (!ifs.is_open()) return false;
	string valueString;
	bool result = getValueString(name, valueString);
	if (result)
	{
		stringstream converter(valueString);
		converter >> value;
		//result = converter.good();
	}
	return result;
}

bool CReaderASCII::read(const string& name, int& value)
{
	if (!ifs.is_open()) return false;
	string valueString;
	bool result = getValueString(name, valueString);
	if (result)
	{
		stringstream converter(valueString);
		converter >> value;
		//result = converter.good();
	}
	return result;
}

bool CReaderASCII::read(const string& name, CMatrix4D& value)
{
	if (!ifs.is_open()) return false;
	bool result = true;
	return result;
}

bool CReaderASCII::read(const string& name, CVector3D& value)
{
	if (!ifs.is_open()) return false;
	string valueString;
	bool result = getValueString(name, valueString);
	if (result) result = value.fromString(valueString);
	return result;
}

bool CReaderASCII::read(const string& name, CColor& value)
{
	if (!ifs.is_open()) return false;
	string valueString;
	bool result = getValueString(name, valueString);
	if (result) result = value.fromString(valueString);
	return result;
}

bool CReaderASCII::read(const string& name, CUVCoord& value)
{
	if (!ifs.is_open()) return false;
	string valueString;
	bool result = getValueString(name, valueString);
	if (result) result = value.fromString(valueString);
	return result;
}

bool CReaderASCII::read(const string& name, string& value)
{
	if (!ifs.is_open()) return false;
	bool result = getValueString(name, value);
	return result;
}

bool CReaderASCII::findLine(const string& patern, string& line)
{
	string temp;
	bool result = false;
	while (!result && !ifs.eof())
	{
		char buf[256];
		ifs.getline(buf, 256);
		if (ifs.good())
		{
			temp = buf;
			result = temp.find(patern) != temp.npos;
		}
	}
	if (result)
	{
		line = temp;
	}
	return result;
}

bool CReaderASCII::getValueString(const string& name, string& valueString)
{
	string line;
	bool result = findLine(name, line);
	if (!result) return result;
	string temp;
	int index = line.find_first_of(name.c_str());
	if (index != line.npos)
	{
		temp = line.substr(index, line.length() - index);
		index = temp.find_first_of(" ");
		if (index != temp.npos)
		{
			valueString = temp.substr(index + 1, temp.length() - index - 1);
			result = true;
		}
	}
	return result;
}
