/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#include "E3D_CWriterASCII.h"
#include "E3D_CMatrix4D.h"
#include "E3D_CVector3D.h"
#include "E3D_CColor.h"
#include "E3D_CUVCoord.h"

#include <iostream>
#include <sstream>
using std::endl;
using std::stringstream;

bool CWriterASCII::open(const string& file)
{
	bool result = false;
	if (close())
	{
		ofs.open(file.c_str());
		result = ofs.is_open();
	}
	return result;
}

bool CWriterASCII::close()
{
	bool result = true;
	if (ofs.is_open())
	{
		ofs.close();
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::push(const string& name)
{
	if (!ofs.is_open()) return false;
	bool result;
	if (result = writeTabs())
	{
		levels++;
		ofs << "begin " << name.c_str() << endl;
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::pop(const string& name)
{
	if (!ofs.is_open() || levels == 0) return false;
	bool result;
	levels--;
	if (result = writeTabs())
	{
		ofs << "end " << name.c_str() << endl;
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::write(const string& name)
{
	if (!ofs.is_open()) return false;
	bool result = true;
	return result;
}

bool CWriterASCII::write(const string& name, const float& value)
{
	if (!ofs.is_open()) return false;
	bool result;
	if (result = writeTabs())
	{
		stringstream converter;
		converter << value;
		ofs << name.c_str() << " " << converter.str() << endl;
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::write(const string& name, const int& value)
{
	if (!ofs.is_open()) return false;
	bool result;
	if (result = writeTabs())
	{
		stringstream converter;
		converter << value;
		ofs << name.c_str() << " " << converter.str() << endl;
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::write(const string& name, const CMatrix4D& value)
{
	if (!ofs.is_open()) return false;
	bool result;
	if (result = writeTabs())
	{
		ofs << name.c_str() << " " << ((string)value).c_str() << endl;
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::write(const string& name, const CVector3D& value)
{
	if (!ofs.is_open()) return false;
	bool result;
	if (result = writeTabs())
	{
		ofs << name.c_str() << " " << ((string)value).c_str() << endl;
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::write(const string& name, const CColor& value)
{
	if (!ofs.is_open()) return false;
	bool result;
	if (result = writeTabs())
	{
		ofs << name.c_str() << " " << ((string)value).c_str() << endl;
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::write(const string& name, const CUVCoord& value)
{
	if (!ofs.is_open()) return false;
	bool result;
	if (result = writeTabs())
	{
		ofs << name.c_str() << " " << ((string)value).c_str() << endl;
		result = ofs.good();
	}
	return result;
}

bool CWriterASCII::write(const string& name, const string& value)
{
	if (!ofs.is_open()) return false;
	bool result;
	if (result = writeTabs())
	{
		ofs << name.c_str() << " " << value.c_str() << endl;
		result = ofs.good();
	}
	return result;
}

CWriterASCII::CWriterASCII(): levels(0)
{
}

CWriterASCII::~CWriterASCII()
{
	close();
}

bool CWriterASCII::writeTabs()
{
	if (!ofs.is_open()) return false;
	bool result = true;
	for (int i=0; i<levels && result; i++)
	{
		ofs << "\t";
		result = ofs.good();
	}
	return result;
}
