/*  Eside 3D library, version 5.
    Copyright (C) 2002 Iker Jamardo Zugaza <ijamardo@eside.deusto.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    This library has been slightly modified by Grzegorz Adam Hankiewicz
    <gradha@terra.es> and is therefor not exactly the original, which you
    should be able to get from Iker by email.
*/

#ifndef _E3D_CINPUTDEVICEKEYBOARDDX8_H_
#define _E3D_CINPUTDEVICEKEYBOARDDX8_H_

#pragma warning (disable: 4786 4503)

#include "E3D_CInputDevice.h"

class CInputDeviceKeyboardDX8: public CInputDevice
{
public:
	virtual void update(float time);
	virtual void setEnabled(bool enabled);

protected:
	enum BUTTON_CODE_DX8
	{
		BUTTON_CODE_DX8_NOBUTTON				= 0x0,
		BUTTON_CODE_DX8_BACK_SPACE			= 14, 
		BUTTON_CODE_DX8_TAB							= 15,
		BUTTON_CODE_DX8_RETURN					= 28,
		BUTTON_CODE_DX8_INTRO						= 156,
		BUTTON_CODE_DX8_ESCAPE					= 1, 
		BUTTON_CODE_DX8_CAPS_LOCK				= 58, 

		BUTTON_CODE_DX8_SHIFT_RIGHT			= 54,
		BUTTON_CODE_DX8_SHIFT_LEFT			= 42,
		BUTTON_CODE_DX8_CONTROL_RIGHT		= 157,
		BUTTON_CODE_DX8_CONTROL_LEFT		= 29,
		BUTTON_CODE_DX8_ALT_RIGHT				= 184,
		BUTTON_CODE_DX8_ALT_LEFT				= 56,
		BUTTON_CODE_DX8_MENU						= 221, 
		BUTTON_CODE_DX8_LWIN						= 219, 
		BUTTON_CODE_DX8_RWIN						= 220, 
		BUTTON_CODE_DX8_NN							= 39, 
                          
		BUTTON_CODE_DX8_OPEN_BRANCH			= 40,
		BUTTON_CODE_DX8_CLOSE_BRANCH		= 43, 
		BUTTON_CODE_DX8_ARROWS					= 86, 
		BUTTON_CODE_DX8_COMMA						= 51, 
		BUTTON_CODE_DX8_MINUS						= 53, 
		BUTTON_CODE_DX8_POINT						= 52, 
		BUTTON_CODE_DX8_SLASH						= 41, 
		BUTTON_CODE_DX8_BACK_QUOTE			= 13, 
		BUTTON_CODE_DX8_OPEN_BRACKET		= 26, 
		BUTTON_CODE_DX8_CLOSE_BRACKET		= 27, 
		BUTTON_CODE_DX8_QUOTE						= 12, 
		BUTTON_CODE_DX8_SPACE						= 57,

		BUTTON_CODE_DX8_PAGE_UP					= 201, 
		BUTTON_CODE_DX8_PAGE_DOWN				= 209, 
		BUTTON_CODE_DX8_END							= 207,
		BUTTON_CODE_DX8_HOME						= 199,
		BUTTON_CODE_DX8_LEFT						= 203,
		BUTTON_CODE_DX8_UP							= 200,
		BUTTON_CODE_DX8_RIGHT						= 205,
		BUTTON_CODE_DX8_DOWN						= 208,
		BUTTON_CODE_DX8_INSERT					= 210,
		BUTTON_CODE_DX8_DELETE					= 211,

		BUTTON_CODE_DX8_0								= 11,
		BUTTON_CODE_DX8_1								= 2,
		BUTTON_CODE_DX8_2								= 3,
		BUTTON_CODE_DX8_3								= 4,
		BUTTON_CODE_DX8_4								= 5,
		BUTTON_CODE_DX8_5								= 6,
		BUTTON_CODE_DX8_6								= 7,
		BUTTON_CODE_DX8_7								=	8,
		BUTTON_CODE_DX8_8								= 9,
		BUTTON_CODE_DX8_9								= 10,

		BUTTON_CODE_DX8_A								= 30,
		BUTTON_CODE_DX8_B								= 48,
		BUTTON_CODE_DX8_C								= 46,
		BUTTON_CODE_DX8_D								= 32,
		BUTTON_CODE_DX8_E								= 18,
		BUTTON_CODE_DX8_F								= 33,
		BUTTON_CODE_DX8_G								= 34,
		BUTTON_CODE_DX8_H								= 35,
		BUTTON_CODE_DX8_I								= 23,
		BUTTON_CODE_DX8_J								= 36,
		BUTTON_CODE_DX8_K								= 37,
		BUTTON_CODE_DX8_L								= 38,
		BUTTON_CODE_DX8_M								= 50,
		BUTTON_CODE_DX8_N								= 49,
		BUTTON_CODE_DX8_O								= 24,
		BUTTON_CODE_DX8_P								= 25,
		BUTTON_CODE_DX8_Q								=	16,
		BUTTON_CODE_DX8_R								= 19,
		BUTTON_CODE_DX8_S								= 31,
		BUTTON_CODE_DX8_T								= 20,
		BUTTON_CODE_DX8_U								= 22,
		BUTTON_CODE_DX8_V								= 47,
		BUTTON_CODE_DX8_W								= 17,
		BUTTON_CODE_DX8_X								= 45,
		BUTTON_CODE_DX8_Y								= 21,
		BUTTON_CODE_DX8_Z								= 44,

		BUTTON_CODE_DX8_NUMLOCK					= 69, 
		BUTTON_CODE_DX8_NUMPAD0					= 82,
		BUTTON_CODE_DX8_NUMPAD1					= 79,
		BUTTON_CODE_DX8_NUMPAD2					= 80,
		BUTTON_CODE_DX8_NUMPAD3					= 81,
		BUTTON_CODE_DX8_NUMPAD4					= 75,
		BUTTON_CODE_DX8_NUMPAD5					= 76,
		BUTTON_CODE_DX8_NUMPAD6					= 77,
		BUTTON_CODE_DX8_NUMPAD7					= 71,
		BUTTON_CODE_DX8_NUMPAD8					= 72,
		BUTTON_CODE_DX8_NUMPAD9					= 73,
		BUTTON_CODE_DX8_MULTIPLY				= 55,
		BUTTON_CODE_DX8_ADD							= 78,
		BUTTON_CODE_DX8_SUBSTRACT				= 74,
		BUTTON_CODE_DX8_DECIMAL					= 83,
		BUTTON_CODE_DX8_DIVIDE					= 181,

		BUTTON_CODE_DX8_F1							= 59,
		BUTTON_CODE_DX8_F2							= 60,
		BUTTON_CODE_DX8_F3							= 61,
		BUTTON_CODE_DX8_F4							= 62,
		BUTTON_CODE_DX8_F5							=	63,
		BUTTON_CODE_DX8_F6							= 64,
		BUTTON_CODE_DX8_F7							= 65,
		BUTTON_CODE_DX8_F8							= 66,
		BUTTON_CODE_DX8_F9							= 67,
		BUTTON_CODE_DX8_F10							= 68,
		BUTTON_CODE_DX8_F11							= 87,
		BUTTON_CODE_DX8_F12							= 88,

		BUTTON_CODE_DX8_PRINTSCREEN			= 183, 
		BUTTON_CODE_DX8_SCROLL_LOCK			= 70, 
		BUTTON_CODE_DX8_PAUSE						= 197
	};
};

#endif