/*  Katapult, version 1.
    Copyright (C) 2002 Grzegorz Adam Hankiewicz <gradha@terra.es> and
    Jon Lpez de Dicastillo <darwater@hotmail.com>.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef KAMERA_H
#define KAMERA_H

#include "E3D_CVector3D.h"

// Wrapper around common camera manipulations
class CScene;
class CCamera;
class CTransform;
class CMesh;

class CKamera
{
	private:
		CTransform *cameraParent, *cameraChild; // position transforms
		CCamera *camera; // the camera itself
		CVector3D pos; // contains the world camera position

		// travel variables
		CVector3D startPos, startTarget, endPos, endTarget;
		float startTime, endTime;
		int endReturnValue;

		void updateAngles(void);

	public:
		CKamera(CScene &scene);
		~CKamera();

		// place the camera in some absolute coordinates position
		void move(float x, float y, float z);
		void move(const CVector3D &pos);
      
		// make the camera point at something
		void point(float x, float y, float z);
		void point(const CVector3D &pos);

		// both operations at the same time
		void moveAndPoint(float x1, float y1, float z1, float x2, float y2, float z2);
		void moveAndPoint(const CVector3D &camera, const CVector3D &target);

		// return position/looking at
		CVector3D position(void);
		CVector3D pointing(void);

		// used to store initial trayectory values
		void computeTrayectory(const CVector3D &startPos, 
			const CVector3D &startTarget, float startTime,
			const CVector3D &endPos, const CVector3D &endTarget,
			float endTime, int endReturnValue);
		// like the other but using actual camera values as start
		void computeTrayectory(float startTime,
			const CVector3D &endPos, const CVector3D &endTarget,
			float endTime, int endReturnValue);
		// place the camera in point of the trayectory depending on time
		int travelTrayectory(float time);

		void putCameraBehindPlayer(const CVector3D &karro1, const CVector3D &karro2, CVector3D &position, CVector3D &target);
		CVector3D interpolateVector(float pos, const CVector3D &p1, const CVector3D &p2);
};

#endif