/*  Katapult, version 1.
    Copyright (C) 2002 Grzegorz Adam Hankiewicz <gradha@terra.es> and
    Jon Lpez de Dicastillo <darwater@hotmail.com>.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Karro.h"

#include "E3D_CMesh.h"
#include "E3D_CScene.h"
#include "E3D_CMath.h"
#include "E3D_CVector3D.h"
#include "E3D_CBoundingBox.h"

#include <assert.h>

/* Creates a Karro. The Karro needs a pointer to the scene in order
 * to insert itself. Also, since each created Karro needs a different
 * name, you have to supply this here.
 */
CKarro::CKarro(CScene &scene, const string &nombre, int zone)
{
	assert(zone == 0 || zone == 1);

	const E3D_MODE mode = E3D_LINES;
	this->nombre = nombre;
	this->z = zone;

	// Definicin de la Catapulta
	tBase = new CTransform();
	tBase->setName(nombre + "Catapulta");
	CVector3D rot = CVector3D(float(0.5*CMath::PI),0,0);
	if(zone) rot.z = CMath::PI;
	tBase->setRotate(rot);
	meshBase = new CMesh();
	meshBase->setName(nombre + "MallaBase");

#define DO_VERTEX(X,Y,Z,R,G,B) meshBase->addVertex(\
	CVector3D(X*0.5f,Y*0.5f,Z*0.5f),\
	CVector3D(0,0,0), CUVCoord(1,1), CColor(R,G,B,1))

	DO_VERTEX(3,1,7,	0,0,0);		// Vertex 0
	DO_VERTEX(3,0,7,	0,0,0);		// Vertex 1
	DO_VERTEX(3,0,-3,	1,1,1);		// Vertex 2
	DO_VERTEX(3,1,-3,	1,1,1);		// Vertex 3
	DO_VERTEX(-3,1,-3,	1,1,1);		// Vertex 4
	DO_VERTEX(-3,1,7,	1,1,1);		// Vertex 5
	DO_VERTEX(-3,0,7,	1,1,1);		// Vertex 6
	DO_VERTEX(-3,0,-3,	1,1,1);		// Vertex 7
	DO_VERTEX(1,1,1,	1,1,1);		// Vertex 8
	DO_VERTEX(1,1,-1,	1,1,1);		// Vertex 9
	DO_VERTEX(-1,1,-1,	1,1,1);		// Vertex 10
	DO_VERTEX(-1,1,1,	1,1,1);		// Vertex 11
	DO_VERTEX(0,3,0,	1,1,1);		// Vertex 12
#undef DO_VERTEX

#define DO_QUAD(P1,P2,P3,P4) \
	polygon.addPositionIndex(P1);\
	polygon.addPositionIndex(P2);\
	polygon.addPositionIndex(P3);\
	polygon.addPositionIndex(P4);\
	polygon.addPositionIndex(P1);\
	meshBase->addPolygon(polygon);\
	polygon.clear()

#define DO_TRI(P1,P2,P3) \
	polygon.addPositionIndex(P1);\
	polygon.addPositionIndex(P2);\
	polygon.addPositionIndex(P3);\
	polygon.addPositionIndex(P1);\
	meshBase->addPolygon(polygon);\
	polygon.clear();

	CPolygon polygon;
	DO_QUAD(0,1,2,3);
	DO_QUAD(3,2,7,4);
	DO_QUAD(0,5,6,1);
	DO_QUAD(4,7,6,5);
	DO_QUAD(1,6,7,2);
	DO_QUAD(0,3,4,5);
	DO_TRI(8,9,12);
	DO_TRI(9,10,12);
	DO_TRI(10,11,12);
	DO_TRI(11,8,12);
#undef DO_QUAD
#undef DO_TRI

	meshBase->setMode(mode);

	// Definicion de la pala de la catapulta
	tPala = new CTransform();
	tPala->setTranslate(CVector3D(0, 1.5f, 0));
	tPala->setName(nombre+"Pala");
	meshPala = new CMesh();
	meshPala->setName(nombre + "MallaPala");

#define DO_VERTEX(X,Y,Z,R,G,B) meshPala->addVertex(\
	CVector3D(X*0.5f,Y*0.5f,Z*0.5f),\
	CVector3D(0,0,0), CUVCoord(1,1), CColor(R,G,B,1))

	DO_VERTEX(1,0,-1,	0,0,0);		// Vertex 0
	DO_VERTEX(1,1,-1,	0,0,0);		// Vertex 1
	DO_VERTEX(-1,0,-1,	1,1,1);		// Vertex 2
	DO_VERTEX(-1,1,-1,	1,1,1);		// Vertex 3
	DO_VERTEX(1,0,3,	1,1,1);		// Vertex 4
	DO_VERTEX(1,1,3,	1,1,1);		// Vertex 5
	DO_VERTEX(-1,0,3,	1,1,1);		// Vertex 6
	DO_VERTEX(-1,1,3,	1,1,1);		// Vertex 7
	DO_VERTEX(1.5f,0,3,	1,1,1);		// Vertex 8
	DO_VERTEX(1.5f,1,3,	1,1,1);		// Vertex 9
	DO_VERTEX(-1.5f,0,3,1,1,1);		// Vertex 10
	DO_VERTEX(-1.5f,1,3,1,1,1);		// Vertex 11
	DO_VERTEX(1.5f,0,6,	1,1,1);		// Vertex 12
	DO_VERTEX(1.5f,1,6,	1,1,1);		// Vertex 13
	DO_VERTEX(-1.5f,0,6,1,1,1);		// Vertex 14
	DO_VERTEX(-1.5f,1,6,1,1,1);		// Vertex 15
#undef DO_VERTEX

#define DO_QUAD(P1,P2,P3,P4) \
	polygon.addPositionIndex(P1);\
	polygon.addPositionIndex(P2);\
	polygon.addPositionIndex(P3);\
	polygon.addPositionIndex(P4);\
	polygon.addPositionIndex(P1);\
	meshPala->addPolygon(polygon);\
	polygon.clear()

	DO_QUAD(0,2,3,1);
	DO_QUAD(4,0,1,5);
	DO_QUAD(6,7,3,2);
	DO_QUAD(5,1,3,7);
	DO_QUAD(4,6,2,0);
	DO_QUAD(9,8,4,5);
	DO_QUAD(6,10,11,7);
	DO_QUAD(8,9,13,12);
	DO_QUAD(10,14,15,11);
	DO_QUAD(15,14,12,13);
	DO_QUAD(9,11,15,13);
	DO_QUAD(8,12,14,10);
#undef DO_QUAD
	
	meshPala->setMode(mode);

	// Definiciones para el proyecytil
	tProy = new CTransform();
	tProy->setTranslate(CVector3D(0,1,2.25f));
	tProy->setName (nombre + "Proyectil");
	meshProy = new CMesh();
	meshProy->setName(nombre + "MallaProyectil");
	meshProy->setMode(mode);

#define DO_VERTEX(X,Y,Z,R,G,B) meshProy->addVertex(\
	CVector3D(X*0.5f,Y*0.5f,Z*0.5f),\
	CVector3D(0,0,0), CUVCoord(1,1), CColor(R,G,B,1))

	DO_VERTEX( 1, 1, 1, 0,0,0);		// Vertex 0
	DO_VERTEX(-1, 1, 1, 0,0,0);		// Vertex 1
	DO_VERTEX(-1, 1,-1, 1,1,1);		// Vertex 2
	DO_VERTEX( 1, 1,-1, 1,1,1);		// Vertex 3
	DO_VERTEX( 1,-1, 1, 1,1,1);		// Vertex 4
	DO_VERTEX(-1,-1, 1, 1,1,1);		// Vertex 5
	DO_VERTEX(-1,-1,-1, 1,1,1);		// Vertex 6
	DO_VERTEX( 1,-1,-1, 1,1,1);		// Vertex 7
#undef DO_VERTEX


#define DO_QUAD(P1,P2,P3,P4) \
	polygon.addPositionIndex(P1);\
	polygon.addPositionIndex(P2);\
	polygon.addPositionIndex(P3);\
	polygon.addPositionIndex(P4);\
	polygon.addPositionIndex(P1);\
	meshProy->addPolygon(polygon);\
	polygon.clear()

	DO_QUAD(0,3,2,1);
	DO_QUAD(4,5,6,7);
	DO_QUAD(1,2,6,5);
	DO_QUAD(0,4,7,3);
	DO_QUAD(2,3,7,6);
	DO_QUAD(0,1,5,4);
#undef DO_QUAD

	scene.addNode(tBase);
	scene.addNode(tPala);
	scene.addNode(meshBase);
	scene.addNode(meshPala);
	scene.addNode(tProy);
	scene.addNode(meshProy);

	scene.addChild(tBase);
	tBase->addChild(meshBase);
	tBase->addChild(tPala);
	tPala->addChild(meshPala);
	tPala->addChild(tProy);
	tProy->addChild(meshProy);
}

CKarro::~CKarro()
{
	delete meshBase;
	delete meshPala;
	delete tBase;
	delete tPala;
}

void CKarro::move(const CVector3D &pos)
{
	tBase->setTranslate(pos);
}

// get actual position
CVector3D CKarro::position(void)
{
	return tBase->getTranslate();
}

// get actual angle rotation
float CKarro::angle(void)
{
	return tPala->getRotate().y;
}

// set actual angle rotation
void CKarro::angle(float angle)
{
	CVector3D p = tPala->getRotate();
	p.y = angle;
	tPala->setRotate(p);
}

/* Each karro has a specific zone to live in (either zero or one).
 */
int CKarro::zone(void)
{
	return z;
}

int CKarro::getRecorrido(float time, CVector3D &posBala, int &freezeCamera)
{
	assert(endTime > startTime);
	assert(time >= 0);

	int next = 0;
		time = (time - startTime) / (endTime - startTime);

	CVector3D rot = tPala->getRotate();
	if (time < 0) {
		tPala->setRotate(CVector3D(-0.25 - 2*time, rot.y, rot.z));
	}
	else
	{
		tPala->setRotate(CVector3D(0, rot.y, rot.z));
		float altura = CMath::sinus(CMath::PI*1.01 * time)* 0.1;

		if(time > 0.8f) 
			freezeCamera = 1;
		else 
			freezeCamera = 0;

		if(time > 1) { // did we end? make sure we reach destination
			next = nextState;
			altura = time = 0;	
		}
		CVector3D retorno = CVector3D( 0, 1+altura * power, 2.25f -power * time);
		tProy->setTranslate(retorno);
	}
	

	CMatrix4D mundo = tProy->getWorldMatrix();
	posBala = CVector3D(mundo.w.x, mundo.w.y, mundo.w.z);

	return next;
}

void CKarro::disparar(float power, float startTime, float endTime, int nextState)
{
	this->power = power;
	this->startTime = startTime;
	this->endTime = endTime;
	this->nextState = nextState;
}

CVector3D CKarro::getPosBala(void)
{
	CMatrix4D mundo = tProy->getWorldMatrix();
	return CVector3D(mundo.w.x, mundo.w.y, mundo.w.z);
}

/*
 */
CBoundingBox CKarro::getBoundingBox(void)
{
	return tBase->getBoundingBox();
}

CBoundingBox CKarro::getBalaBoundingBox(void)
{
	return tProy->getBoundingBox();
}

void CKarro::destruir(float startTime, float endTime, int nextState)
{
	this->startTime = startTime;
	this->endTime = endTime;
	this->nextState = nextState;
}

void CKarro::destruyendo (float time)
{
	time = (time - startTime) / (endTime - startTime);

	tBase->setRotate (CVector3D(0.5*CMath::PI, 0, time));

	tPala->setRotate (CVector3D(0.5*CMath::PI*time, 0, /* */ -time /* */));
	tPala->setTranslate (CVector3D(2*time, 1.5f* (1 - time), 0));

	tProy->setRotate (CVector3D(/* */-0.5*CMath::PI*time/* */, 0, 0));
	tProy->setTranslate (CVector3D(/* */-2*time/* */, /* */1 - 1.5f*(1-time) /* */- 2.5*(1-time), 2.25f));
}