/*  Katapult, version 1.
    Copyright (C) 2002 Grzegorz Adam Hankiewicz <gradha@terra.es> and
    Jon Lpez de Dicastillo <darwater@hotmail.com>.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef KATAPULT_H
#define KATAPULT_H

#include "E3D_CApplication.h"
#include "E3D_CInput.h"
#include "E3D_CScene.h"
#include "E3D_CTimer.h"

class CRender;
class CTransform;
class CKamera;
class CKarro;
class CEarth;
class CBoundingBox;

class Katapult: public CApplication, public IInputListener
{
public:
// From CApplication
	virtual bool init();
	virtual bool update();
	virtual bool end();

	virtual bool eventResize(int width, int height);

// From IInputListener
	virtual void inputChanged(CInputEvent& ie);

// Own
	Katapult();

protected:
	CInput input;
	CScene scene;
	CTimer timer;
	CRender *render;
	CEarth *earth;
	CKamera *kamera;   // Class holding camera information
	CKarro *karro1, *karro2; // Classes for the katapults
	CVector3D lastCameraPosition;
	int logicState;
	int modeChange, moveX, moveY, fire; // logical movement.

	void updateLogic(void);
	void logicPlayerMove(void);
	void logicCameraToPlayer(void);
	void logicPlayerAim(void);
	void logicFollowBullet(void);
	void putCameraBehindPala(CVector3D &pos, CVector3D &dest);
	bool boundingBoxCollision(const CBoundingBox &p1, const CBoundingBox &p2);
};

// logic states, should be non-zero
enum LOGIC_STATE
{
	FOLLOW_BULLET = 1,
	CAMERA_MOVE = 2,
	BULLET_FLIGHT = 3,
	PLAYER_MOVE = 4,
	PLAYER_AIM = 5,
	PLAYER_SHOOT = 6,
	CAMERA_TO_PLAYER = 10
};

// Defines for the size of the "arena"
#define WORLD_WIDTH		80
#define WORLD_HEIGHT	100

#define MAX(A,B) (A > B) ? (A) : (B)
#define MIN(A,B) (A < B) ? (A) : (B)

#endif 